@@ -3,6 +3,7 @@
 
 // Shasta.
 #include "AlignmentCandidates.hpp"
+#include "AssemblyGraph2Statistics.hpp"
 #include "HttpServer.hpp"
 #include "Kmer.hpp"
 #include "LocalAlignmentCandidateGraph.hpp"
@@ -32,7 +33,6 @@ namespace shasta {
     class AssemblerOptions;
     class AssembledSegment;
     class AssemblyGraph2;
-    class Bubbles;
     class CompressedAssemblyGraph;
     class ConsensusCaller;
     class Histogram2;
@@ -40,19 +40,18 @@ namespace shasta {
     class LocalAssemblyGraph;
     class LocalAlignmentCandidateGraph;
     class LocalAlignmentGraph;
+    class LocalMarkerGraph;
     class LocalReadGraph;
     class LocalReadGraphTriangles;
     class LocalMarkerGraphRequestParameters;
     class LongBaseSequences;
     class MarkerConnectivityGraph;
     class MarkerConnectivityGraphVertexMap;
+    class Mode2AssemblyOptions;
     class OrientedReadPair;
     class Reads;
     class ReferenceOverlapMap;
 
-#ifdef SHASTA_HTTP_SERVER
-    class LocalMarkerGraph;
-#endif
 
     namespace MemoryMapped {
         class ByteAllocator;
@@ -69,6 +68,10 @@ namespace shasta {
         class MatrixEntry;
         class Options;
     }
+
+    namespace mode3 {
+        class AssemblyGraph;
+    }
 }
 
 namespace spoa {
@@ -90,12 +93,20 @@ class DisjointSets;
 class shasta::AssemblerInfo {
 public:
 
+    // The read representation used: 0 = raw sequence, 1 = RLE sequence
+    uint64_t readRepresentation;
+
     // The length of k-mers used to define markers.
     size_t k;
 
     // The page size in use for this run.
     size_t largeDataPageSize;
 
+    // Assembly mode (0=haploid, 2=phased).
+    uint64_t assemblyMode;
+
+    // Read graph creation method.
+    uint64_t readGraphCreationMethod;
 
 
     // Statistics on the number of reads discarded on input.
@@ -116,10 +127,6 @@ public:
     uint64_t discardedBadRepeatCountReadCount = 0;
     uint64_t discardedBadRepeatCountBaseCount = 0;
 
-    // The number of reads and raw bases discarded because the read
-    // contained repeat counts greater than 255.
-    uint64_t discardedPalindromicReadCount = 0;
-    uint64_t discardedPalindromicBaseCount = 0;
 
 
     // Statistics for the reads kept in the assembly
@@ -136,12 +143,16 @@ public:
     uint64_t isolatedReadCount = 0;
     uint64_t isolatedReadBaseCount = 0;
 
-    // Alignment criteria for readGraph creation method 2
-    double automatedAlignedFractionThreshold = 0;
-    double automatedMarkerCountThreshold = 0;
-    double automatedMaxDriftThreshold = 0;
-    double automatedMaxSkipThreshold = 0;
-    double automatedMaxTrimThreshold = 0;
+    // Alignment criteria actually used.
+    // For readGraph creation method 0, they are the values specified
+    // by the command line options and/or configuration.
+    // For readGraph creation method 2, they are dynamically selected
+    // based on alignments statistics.
+    double actualMinAlignedFraction = 0;
+    uint64_t actualMinAlignedMarkerCount = 0;
+    uint64_t actualMaxDrift = 0;
+    uint64_t actualMaxSkip = 0;
+    uint64_t actualMaxTrim = 0;
 
     // Marker graph statistics.
     size_t markerGraphVerticesNotIsolatedCount = 0;
@@ -154,6 +165,9 @@ public:
     size_t longestAssembledSegmentLength = 0;
     size_t assembledSegmentN50 = 0;
 
+    // Mode 2 assembly statistics.
+    AssemblyGraph2Statistics assemblyGraph2Statistics;
+
     // Performance statistics.
     double assemblyElapsedTimeSeconds = 0.;
     double averageCpuUtilization;
@@ -193,6 +207,7 @@ public:
     Assembler(
         const string& largeDataFileNamePrefix,
         bool createNew,
+        uint64_t readRepresentation, // 0 = raw sequence, 1 = RLE sequence. Only used if createNew.
         size_t largeDataPageSize);
 
     // Add reads.
@@ -201,11 +216,6 @@ public:
         const string& fileName,
         uint64_t minReadLength,
         bool noCache,
-        bool detectPalindromesOnFastqLoad,
-        double qScoreRelativeMeanDifference,
-        double qScoreMinimumMean,
-        double qScoreMinimumVariance,
-        bool writePalindromicReadsToCsv,
         size_t threadCount);
 
     // Create a histogram of read lengths.
@@ -386,8 +396,8 @@ private:
 
     // Various pieces of assembler information stored in shared memory.
     // See class AssemblerInfo for more information.
-    MemoryMapped::Object<AssemblerInfo> assemblerInfo;
 public:
+    MemoryMapped::Object<AssemblerInfo> assemblerInfo;
     uint64_t getMarkerGraphMinCoverageUsed() const
     {
         return assemblerInfo->markerGraphMinCoverageUsed;
@@ -624,7 +634,6 @@ public:
         double alignedFractionThreshold,
         double nearDiagonalFractionThreshold,
         uint32_t deltaThreshold,
-        bool writeToCsv,
         size_t threadCount);
 private:
     void flagPalindromicReadsThreadFunction(size_t threadId);
@@ -1116,10 +1125,6 @@ public:
 #endif
 
 
-    // Read graph creation for mode 1 assembly.
-    void createReadGraphMode1(uint64_t maxAlignmentCount);
-
-
 
     // Approximate strand separation in the read graph.
     void flagCrossStrandReadGraphEdges1(int maxDistance, size_t threadCount);
@@ -1452,10 +1457,6 @@ public:
     void writeParallelMarkerGraphEdges() const;
 
 
-    // Analyze forks in the marker graph.
-    void analyzeMarkerGraphForks();
-
-
 
     // Function createMarkerGraphSecondaryEdges can be called after createMarkerGraphEdgesStrict
     // to create a minimal amount of additional non-strict edges (secondary edges)
@@ -1468,10 +1469,48 @@ private:
         uint32_t secondaryEdgeMaxSkip,
         bool aggressive,
         size_t threadCount);
-
-
-
 public:
+
+
+
+    // Cluster the oriented reads on a marker graph edge based on their sequence.
+    // This returns a vector of connected components.
+    // Each connected component is an index into the marker intervals for the edge.
+    vector< vector<uint64_t> > clusterMarkerGraphEdgeOrientedReads(
+        MarkerGraphEdgeId,
+        double errorRateThreshold,
+        bool debug) const;
+
+
+
+    // Use clusterMarkerGraphEdgeOrientedReads to split secondary marker graph edges
+    // where necessary.
+    void splitMarkerGraphSecondaryEdges(
+        double errorRateThreshold,
+        uint64_t minCoverage,
+        size_t threadCount);
+    void splitMarkerGraphSecondaryEdgesThreadFunction(size_t threadId);
+    class SplitMarkerGraphSecondaryEdgesData {
+    public:
+        double errorRateThreshold;
+        uint64_t minCoverage;
+        uint64_t initialSecondaryCount;
+        uint64_t splitCount;
+        uint64_t createdCount;
+
+        // The new edges that were created by each thread.
+        class Edge {
+        public:
+            MarkerGraphVertexId source;
+            MarkerGraphVertexId target;
+            vector<MarkerInterval> markerIntervals;
+        };
+        vector< vector<Edge> > threadEdges;
+    };
+    SplitMarkerGraphSecondaryEdgesData splitMarkerGraphSecondaryEdgesData;
+
+
+
     // Set marker graph edge flags to specified values for all marker graph edges.
     // Specify any value other than 0 or 1 leaves that flag unchanged.
     // Only useful for debugging.
@@ -1509,15 +1548,6 @@ public:
     // Prune leaves from the strong subgraph of the global marker graph.
     void pruneMarkerGraphStrongSubgraph(size_t iterationCount);
 
-    // Refine the marker graph by removing vertices in tangle regions,
-    // then recreating edges. This must be called after
-    // transitive reduction. After this is called, the only
-    // two MarkerGraph field filled in are vertices and vertexTable.
-    // Everything else has to be recreated.
-    void refineMarkerGraph(
-        uint64_t refineThreshold,
-        size_t threadCount);
-
 private:
 
 
@@ -1665,7 +1695,6 @@ private:
 
 
 
-#ifdef SHASTA_HTTP_SERVER
     // Extract a local subgraph of the global marker graph.
     bool extractLocalMarkerGraph(
         OrientedReadId,
@@ -1678,6 +1707,7 @@ private:
         bool usePrunedEdges,
         bool useSuperBubbleEdges,
         bool useLowCoverageCrossEdges,
+        bool useRemovedSecondaryEdges,
         LocalMarkerGraph&
         );
     bool extractLocalMarkerGraph(
@@ -1690,9 +1720,9 @@ private:
         bool usePrunedEdges,
         bool useSuperBubbleEdges,
         bool useLowCoverageCrossEdges,
+        bool useRemovedSecondaryEdges,
         LocalMarkerGraph&
         );
-#endif
 
     // Compute consensus sequence for a vertex of the marker graph.
     void computeMarkerGraphVertexConsensusSequence(
@@ -2128,9 +2158,9 @@ public:
     // of processing requests.
     void writeHtmlBegin(ostream&) const;
     void writeHtmlEnd(ostream&) const;
-    void writeAssemblySummary(ostream&, bool readsOnly=false);
-    void writeAssemblySummaryBody(ostream&, bool readsOnly=false);
-    void writeAssemblySummaryJson(ostream&, bool readsOnly=false);
+    void writeAssemblySummary(ostream&);
+    void writeAssemblySummaryBody(ostream&);
+    void writeAssemblySummaryJson(ostream&);
     void writeAssemblyIndex(ostream&) const;
     static void writeStyle(ostream& html);
 
@@ -2161,6 +2191,8 @@ public:
         const BrowserInformation&) override;
     void exploreSummary(const vector<string>&, ostream&);
     void exploreRead(const vector<string>&, ostream&);
+    void exploreReadRaw(const vector<string>&, ostream&);
+    void exploreReadRle(const vector<string>&, ostream&);
     void blastRead(const vector<string>&, ostream&);
     void exploreAlignmentCandidateGraph(const vector<string>& request, ostream& html);
     void exploreAlignments(const vector<string>&, ostream&);
@@ -2342,6 +2374,13 @@ private:
     void exploreAssemblyGraphEdgesSupport(const vector<string>&, ostream&);
 
 
+    // Http server functionality specific to mode 3 assembly.
+    void exploreMode3AssemblyGraph(const vector<string>&, ostream&);
+    void exploreMode3AssemblyGraphSegment(const vector<string>&, ostream&);
+    void exploreMode3AssemblyGraphSegmentPair(const vector<string>&, ostream&);
+    void exploreMode3AssemblyGraphLink(const vector<string>&, ostream&);
+
+
 
     // Set up the ConsensusCaller used to compute the "best"
     // base and repeat count at each assembly position.
@@ -2360,31 +2399,25 @@ public:
     void setupConsensusCaller(const string&);
 private:
     shared_ptr<ConsensusCaller> consensusCaller;
-
-
-
-    // Bubbles in the initial assembly graph.
-    // Only used in assembly mode 1.
-    shared_ptr<Bubbles> bubbles = 0;
 public:
-    void analyzeAssemblyGraphBubbles(bool debug = false);
 
 
 
     // Assembly graph for mode 2 assembly.
     shared_ptr<AssemblyGraph2> assemblyGraph2Pointer;
     void createAssemblyGraph2(
-        double bubbleRemovalDiscordantRatioThreshold,
-        double bubbleRemovalAmbiguityThreshold,
-        uint64_t bubbleRemovalMaxPeriod,
-        uint64_t superbubbleRemovalEdgeLengthThreshold,
-        uint64_t phasingMinReadCount,
-        bool suppressGfaOutput,
-        bool suppressFastaOutput,
-        bool suppressDetailedOutput,
-        bool suppressPhasedOutput,
-        bool suppressHaploidOutput,
+        uint64_t pruneLength,
+        const Mode2AssemblyOptions&,
+        size_t threadCount,
+        bool debug);
+
+
+    // Mode 3 assembly.
+    void mode3Assembly(
         size_t threadCount);
+    shared_ptr<mode3::AssemblyGraph> assemblyGraph3Pointer;
+    void accessMode3AssemblyGraph();
+    void analyzeMode3Subgraph(const vector<uint64_t>& segmentIds);
 
 
 
