@@ -7,6 +7,7 @@
 #include "Align4.hpp"
 #include "AssemblerOptions.hpp"
 #include "compressAlignment.hpp"
+#include "performanceLog.hpp"
 #include "Reads.hpp"
 #include "span.hpp"
 #include "timestamp.hpp"
@@ -212,8 +213,8 @@ void Assembler::computeAlignments(
 )
 {
     const auto tBegin = steady_clock::now();
-    cout << timestamp << "Begin computing alignments for ";
-    cout << alignmentCandidates.candidates.size() << " alignment candidates." << endl;
+    performanceLog << timestamp << "Begin computing alignments for ";
+    performanceLog << alignmentCandidates.candidates.size() << " alignment candidates." << endl;
 
     // Check that we have what we need.
     reads->checkReadsAreOpen();
@@ -250,13 +251,13 @@ void Assembler::computeAlignments(
     data.threadAlignmentData.resize(threadCount);
     data.threadCompressedAlignments.resize(threadCount);
     
-    cout << timestamp << "Alignment computation begins." << endl;
+    performanceLog << timestamp << "Alignment computation begins." << endl;
     setupLoadBalancing(alignmentCandidates.candidates.size(), batchSize);
     runThreads(&Assembler::computeAlignmentsThreadFunction, threadCount);
-    cout << timestamp << "Alignment computation completed." << endl;
+    performanceLog << timestamp << "Alignment computation completed." << endl;
 
     // Store the alignments found by each thread.
-    cout << timestamp << "Storing the alignment found by each thread." << endl;
+    performanceLog << timestamp << "Storing the alignment found by each thread." << endl;
     alignmentData.createNew(largeDataName("AlignmentData"), largeDataPageSize);
     compressedAlignments.createNew(largeDataName("CompressedAlignments"), largeDataPageSize);
     
@@ -289,15 +290,15 @@ void Assembler::computeAlignments(
     }
 
     cout << "Found and stored " << alignmentData.size() << " good alignments." << endl;
-    cout << timestamp << "Creating alignment table." << endl;
+    performanceLog << timestamp << "Creating alignment table." << endl;
     computeAlignmentTable();
 
     const auto tEnd = steady_clock::now();
     const double tTotal = seconds(tEnd - tBegin);
-    cout << timestamp << "Computation of alignments ";
-    cout << "completed in " << tTotal << " s." << endl;
+    performanceLog << timestamp << "Computation of alignments ";
+    performanceLog << "completed in " << tTotal << " s." << endl;
 
-    cout << timestamp;
+    performanceLog << timestamp;
 }
 
 
@@ -366,8 +367,8 @@ void Assembler::computeAlignmentsThreadFunction(size_t threadId)
     while(getNextBatch(begin, end)) {
         if((begin % 1000000) == 0){
             std::lock_guard<std::mutex> lock(mutex);
-            cout << timestamp << "Working on alignment " << begin;
-            cout << " of " << alignmentCandidates.candidates.size() << endl;
+            performanceLog << timestamp << "Working on alignment " << begin;
+            performanceLog << " of " << alignmentCandidates.candidates.size() << endl;
         }
 
         for(size_t i=begin; i!=end; i++) {
@@ -653,10 +654,9 @@ void Assembler::flagPalindromicReads(
     double alignedFractionThreshold,
     double nearDiagonalFractionThreshold,
     uint32_t deltaThreshold,
-    bool writeToCsv,
     size_t threadCount)
 {
-    cout << timestamp << "Finding palindromic reads." << endl;
+    performanceLog << timestamp << "Finding palindromic reads." << endl;
 
     // Adjust the numbers of threads, if necessary.
     if(threadCount == 0) {
@@ -690,26 +690,11 @@ void Assembler::flagPalindromicReads(
         }
     }
     assemblerInfo->palindromicReadCount = palindromicReadCount;
-    cout << timestamp << "Flagged " << palindromicReadCount <<
+    cout << "Flagged " << palindromicReadCount <<
         " reads as palindromic out of " << readCount << " total." << endl;
     cout << "Palindromic fraction is " <<
         double(palindromicReadCount)/double(readCount) << endl;
 
-
-    // Write a csv file with the list of palindromic reads
-    // if shasta is running in "filterReads" mode.
-    // This should not too big as the typical rate of
-    // palindromic reads is around 1e-4.
-    if(writeToCsv) {
-        // Open the file in "append" mode because it may already have
-        // been created during read loading
-        ofstream csvOut("PalindromicReads.csv", std::ofstream::app);
-        for(ReadId readId = 0; readId < readCount; readId++) {
-            if(reads->getFlags(readId).isPalindromic) {
-                csvOut << reads->getReadName(readId) << "\n";
-            }
-        }
-    }
 }
 
 
@@ -736,13 +721,8 @@ void Assembler::flagPalindromicReadsThreadFunction(size_t threadId)
     // Loop over all batches assigned to this thread.
     uint64_t begin, end;
     reads->assertReadsAndFlagsOfSameSize();
-    ReadId readCount = reads->readCount();
 
     while(getNextBatch(begin, end)) {
-        if((begin%1000000) == 0) {
-            std::lock_guard<std::mutex> lock(mutex);
-            cout << timestamp << begin << "/" << readCount << endl;
-        }
 
         // Loop over all reads in this batch.
         for(ReadId readId=ReadId(begin); readId!=ReadId(end); readId++) {
@@ -1187,11 +1167,10 @@ void Assembler::suppressAlignmentCandidates(
     uint64_t delta,
     size_t threadCount)
 {
-    cout << timestamp << "Suppressing alignment candidates." << endl;
+    performanceLog << timestamp << "Suppressing alignment candidates." << endl;
 
     // Allocate memory for flags to keep track of which alignments
     // should be suppressed.
-
     suppressAlignmentCandidatesData.suppress.createNew(
         largeDataName("tmp-suppressAlignmentCandidates"), largeDataPageSize);
     const uint64_t candidateCount = alignmentCandidates.candidates.size();
@@ -1233,7 +1212,7 @@ void Assembler::suppressAlignmentCandidates(
     // Clean up.
     suppressAlignmentCandidatesData.suppress.remove();
 
-    cout << timestamp << "Done suppressing alignment candidates." << endl;
+    performanceLog << timestamp << "Done suppressing alignment candidates." << endl;
 }
 
 
