@@ -60,11 +60,6 @@ void Assembler::exploreMarkerGraph(
 
 
 
-
-
-
-
-
     // Validity checks.
     if(requestParameters.vertexId > markerGraph.vertexCount()) {
         html << "<p>Invalid vertex id " << requestParameters.vertexId;
@@ -76,7 +71,9 @@ void Assembler::exploreMarkerGraph(
 
     // Create the local marker graph.
     LocalMarkerGraph graph(
+        assemblerInfo->readRepresentation,
         uint32_t(assemblerInfo->k),
+        assemblerInfo->assemblyMode,
         getReads(),
         markers,
         markerGraph.vertexTable,
@@ -92,6 +89,7 @@ void Assembler::exploreMarkerGraph(
         requestParameters.usePrunedEdges,
         requestParameters.useSuperBubbleEdges,
         requestParameters.useLowCoverageCrossEdges,
+        requestParameters.useRemovedSecondaryEdges,
         graph)) {
         html << "<p>Timeout for graph creation exceeded. Increase the timeout or reduce the maximum distance from the start vertex.";
         return;
@@ -208,11 +206,11 @@ void Assembler::exploreMarkerGraph(
     if(requestParameters.edgeColoring == "byFlags") {
         if(requestParameters.highlightedOrientedReads.empty()) {
             html << "<h3>Color legend for edges lines and arrows</h3>";
-            LocalMarkerGraph::writeColorLegendEdgeArrowsByFlags(html);
+            graph.writeColorLegendEdgeArrowsByFlags(html);
         }
         if(requestParameters.edgeLabels > 0) {
             html << "<h3>Color legend for edge labels</h3>";
-            LocalMarkerGraph::writeColorLegendEdgeLabelsByFlags(html);
+            graph.writeColorLegendEdgeLabelsByFlags(html);
         }
     }
 
@@ -404,6 +402,10 @@ void Assembler::getLocalMarkerGraphRequestParameters(
     parameters.useLowCoverageCrossEdges = getParameterValue(
         request, "useLowCoverageCrossEdges", useLowCoverageCrossEdgesString);
 
+    string useRemovedSecondaryEdgesString;
+    parameters.useRemovedSecondaryEdges = getParameterValue(
+        request, "useRemovedSecondaryEdges", useRemovedSecondaryEdgesString);
+
     parameters.sizePixels = 800;
     parameters.sizePixelsIsPresent = getParameterValue(
         request, "sizePixels", parameters.sizePixels);
@@ -560,6 +562,13 @@ void LocalMarkerGraphRequestParameters::writeForm(
         "<input type=checkbox name=useLowCoverageCrossEdges" <<
         (useLowCoverageCrossEdges ? " checked=checked" : "") << ">"
 
+        "<tr title='Check to include in the local marker graph "
+        "secondary edges that were removed during splitting'>"
+        "<td>Secondary edges removed during splitting"
+        "<td class=centered>"
+        "<input type=checkbox name=useRemovedSecondaryEdges" <<
+        (useRemovedSecondaryEdges ? " checked=checked" : "") << ">"
+
         "</table>"
         "<h3>Graphics</h3>"
 
@@ -869,8 +878,13 @@ void Assembler::exploreMarkerGraphVertex(const vector<string>& request, ostream&
         // Get the repeat count for this marker at each of the k positions.
         for(size_t i=0; i<k; i++) {
             Base base;
-            tie(base, repeatCounts[j][i]) =
-                reads->getOrientedReadBaseAndRepeatCount(orientedReadIds[j], uint32_t(marker.position+i));
+            if(assemblerInfo->readRepresentation == 1) {
+                tie(base, repeatCounts[j][i]) =
+                    reads->getOrientedReadBaseAndRepeatCount(orientedReadIds[j], uint32_t(marker.position+i));
+            } else {
+                base = reads->getOrientedReadBase(orientedReadIds[j], uint32_t(marker.position+i));
+                repeatCounts[j][i] = 1;
+            }
             SHASTA_ASSERT(base == kmer[i]);
         }
     }
@@ -1224,13 +1238,23 @@ void Assembler::exploreMarkerGraphEdge(const vector<string>& request, ostream& h
         const uint32_t positionBegin = marker0.position;
         const uint32_t positionEnd = marker1.position + uint32_t(k);
 
+
+
         // Store the bases and repeat counts in this interval.
         for(uint32_t position=positionBegin; position!=positionEnd; ++position) {
-            Base base;
-            uint8_t repeatCount;
-            tie(base, repeatCount) = reads->getOrientedReadBaseAndRepeatCount(orientedReadId, position);
-            sequences[j].push_back(base);
-            repeatCounts[j].push_back(repeatCount);
+
+            if(assemblerInfo->readRepresentation == 1) {
+                Base base;
+                uint8_t repeatCount;
+                tie(base, repeatCount) = reads->getOrientedReadBaseAndRepeatCount(orientedReadId, position);
+                sequences[j].push_back(base);
+                repeatCounts[j].push_back(repeatCount);
+            } else {
+                const Base base = reads->getOrientedReadBase(orientedReadId, position);
+                sequences[j].push_back(base);
+                repeatCounts[j].push_back(1);
+
+            }
         }
     }
 
@@ -1296,7 +1320,16 @@ void Assembler::exploreMarkerGraphEdge(const vector<string>& request, ostream& h
         "<a href='exploreMarkerGraphVertex?vertexId=" << vertexIds[0] << "'>" << vertexIds[0] << "</a>"
         "<tr><th class=left>Target vertex<td class=centered>" <<
         "<a href='exploreMarkerGraphVertex?vertexId=" << vertexIds[1] << "'>" << vertexIds[1] << "</a>"
-        "<tr><th class=left>Coverage<td class=centered>" << markerCount <<
+        "<tr><th class=left>Coverage<td class=centered>" << markerCount;
+
+    if(assemblerInfo->assemblyMode == 2) {
+        html <<
+        "<tr><th class=left>Secondary edge?<td class=centered>" <<
+        (edge.isSecondary ? "Yes" : "No") <<
+        "<tr><th class=left>Removed while splitting of secondary edges?<td class=centered>" <<
+        (edge.wasRemovedWhileSplittingSecondaryEdges ? "Yes" : "No");
+    } else {
+        html <<
         "<tr><th class=left>Removed during transitive reduction?<td class=centered>" <<
         (edge.wasRemovedByTransitiveReduction ? "Yes" : "No") <<
         "<tr><th class=left>Removed during pruning?<td class=centered>" <<
@@ -1305,6 +1338,7 @@ void Assembler::exploreMarkerGraphEdge(const vector<string>& request, ostream& h
         (edge.isSuperBubbleEdge ? "Yes" : "No") <<
         "<tr><th class=left>Removed as a low coverage cross edge?<td class=centered>" <<
         (edge.isLowCoverageCrossEdge ? "Yes" : "No");
+    }
 
     // Usage of this edge in the assembly graph.
     if(assemblyGraphPointer and assemblyGraphPointer->markerToAssemblyTable.isOpen()) {
