@@ -25,6 +25,19 @@ namespace shasta {
             "index.html",
             "Html file containing a list of assembly output files and their descriptions."
         },
+        {
+            "stdout.log",
+            "Assembly log output."
+        },
+        {
+            "performance.log",
+            "Performance log output. Contains messages that are mnostly useful for "
+            "performance analysis."
+        },
+        {
+            "Data",
+            "Directory containing Shasta binary data."
+        },
         {
             "AssemblySummary.json",
             "Assembly summary information in json format."
@@ -227,6 +240,10 @@ void Assembler::fillServerFunctionTable()
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreAssemblyGraphEdge);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreAssemblyGraphEdgesSupport);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreCompressedAssemblyGraph);
+    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMode3AssemblyGraph);
+    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMode3AssemblyGraphSegment);
+    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMode3AssemblyGraphSegmentPair);
+    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMode3AssemblyGraphLink);
 
 }
 #undef SHASTA_ADD_TO_FUNCTION_TABLE
@@ -434,12 +451,22 @@ void Assembler::writeNavigation(ostream& html) const
         {"Follow a read in the marker graph", "followReadInMarkerGraph"},
         {"Marker connectivity", "exploreMarkerConnectivity"},
         });
-    writeNavigation(html, "Assembly graph", {
-        {"Local assembly graph", "exploreAssemblyGraph"},
-        {"Assembly graph edges", "exploreAssemblyGraphEdge"},
-        {"Assembly graph edges support", "exploreAssemblyGraphEdgesSupport"},
-        {"Compressed assembly graph", "exploreCompressedAssemblyGraph"},
-        });
+    if(assemblerInfo->assemblyMode == 0) {
+        writeNavigation(html, "Assembly graph", {
+            {"Local assembly graph", "exploreAssemblyGraph"},
+            {"Assembly graph edges", "exploreAssemblyGraphEdge"},
+            {"Assembly graph edges support", "exploreAssemblyGraphEdgesSupport"},
+            {"Compressed assembly graph", "exploreCompressedAssemblyGraph"},
+            });
+    }
+    if(assemblerInfo->assemblyMode == 3) {
+        writeNavigation(html, "Assembly graph", {
+            {"Local assembly graph", "exploreMode3AssemblyGraph"},
+            {"Assembly graph segments", "exploreMode3AssemblyGraphSegment"},
+            {"Assembly graph segment pairs", "exploreMode3AssemblyGraphSegmentPair"},
+            {"Assembly graph links", "exploreMode3AssemblyGraphLink"},
+            });
+    }
     
     if (!httpServerData.docsDirectory.empty()) {
         writeNavigation(html, "Help", {
@@ -647,34 +674,6 @@ void Assembler::accessAllSoft()
         allDataAreAvailable = false;
     }
 
-    try {
-        accessAssemblyGraphVertices();
-    } catch(const exception& e) {
-        cout << "Assembly graph vertices are not accessible." << endl;
-        allDataAreAvailable = false;
-    }
-
-    try {
-        accessAssemblyGraphEdges();
-    } catch(const exception& e) {
-        cout << "Assembly graph edges are not accessible." << endl;
-        allDataAreAvailable = false;
-    }
-
-    try {
-        accessAssemblyGraphEdgeLists();
-    } catch(const exception& e) {
-        cout << "Assembly graph edge lists are not accessible." << endl;
-        allDataAreAvailable = false;
-    }
-
-    try {
-        accessAssemblyGraphSequences();
-    } catch(const exception& e) {
-        cout << "Assembly graph sequences are not accessible." << endl;
-        allDataAreAvailable = false;
-    }
-
     try {
         accessCompressedAlignments();
     } catch(const exception& e) {
@@ -682,6 +681,54 @@ void Assembler::accessAllSoft()
         allDataAreAvailable = false;
     }
 
+
+
+    // Data specific to assembly mode 0.
+    if(assemblerInfo->assemblyMode == 0) {
+        try {
+            accessAssemblyGraphVertices();
+        } catch(const exception& e) {
+            cout << "Assembly graph vertices are not accessible." << endl;
+            allDataAreAvailable = false;
+        }
+
+        try {
+            accessAssemblyGraphEdges();
+        } catch(const exception& e) {
+            cout << "Assembly graph edges are not accessible." << endl;
+            allDataAreAvailable = false;
+        }
+
+        try {
+            accessAssemblyGraphEdgeLists();
+        } catch(const exception& e) {
+            cout << "Assembly graph edge lists are not accessible." << endl;
+            allDataAreAvailable = false;
+        }
+
+        try {
+            accessAssemblyGraphSequences();
+        } catch(const exception& e) {
+            cout << "Assembly graph sequences are not accessible." << endl;
+            allDataAreAvailable = false;
+        }
+
+    }
+
+
+
+    // Data specific to assembly mode 3.
+    if(assemblerInfo->assemblyMode == 3) {
+        try {
+            accessMode3AssemblyGraph();
+        } catch(const exception& e) {
+            cout << "The mode 3 assembly graph is not accessible." << endl;
+            allDataAreAvailable = false;
+        }
+    }
+
+
+
     if(!allDataAreAvailable) {
         cout << "Not all assembly data are accessible." << endl;
         cout << "Some functionality is not available." << endl;
@@ -724,6 +771,9 @@ void Assembler::writeStyle(ostream& html)
     td.centered {
         text-align: center;
     }
+    td.left {
+        text-align: left;
+    }
     td.right {
         text-align: right;
     }
@@ -837,14 +887,14 @@ void Assembler::writeHtmlEnd(ostream& html) const
 
 
 
-void Assembler::writeAssemblySummary(ostream& html, bool readsOnly)
+void Assembler::writeAssemblySummary(ostream& html)
 {
     writeHtmlBegin(html);
-    writeAssemblySummaryBody(html, readsOnly);
+    writeAssemblySummaryBody(html);
     writeHtmlEnd(html);
 }
 
-void Assembler::writeAssemblySummaryBody(ostream& html, bool readsOnly)
+void Assembler::writeAssemblySummaryBody(ostream& html)
 {
     using std::setprecision;
     AssemblyGraph& assemblyGraph = *assemblyGraphPointer;
@@ -865,13 +915,11 @@ void Assembler::writeAssemblySummaryBody(ostream& html, bool readsOnly)
     const uint64_t totalDiscardedReadCount =
         assemblerInfo->discardedInvalidBaseReadCount +
         assemblerInfo->discardedShortReadReadCount +
-        assemblerInfo->discardedBadRepeatCountReadCount +
-        assemblerInfo->discardedPalindromicReadCount;
+        assemblerInfo->discardedBadRepeatCountReadCount;
     const uint64_t totalDiscardedBaseCount =
         assemblerInfo->discardedInvalidBaseBaseCount +
         assemblerInfo->discardedShortReadBaseCount +
-        assemblerInfo->discardedBadRepeatCountBaseCount +
-        assemblerInfo->discardedPalindromicBaseCount;
+        assemblerInfo->discardedBadRepeatCountBaseCount;
 
 
     html <<
@@ -883,28 +931,34 @@ void Assembler::writeAssemblySummaryBody(ostream& html, bool readsOnly)
 
         "<h3>Reads used in this assembly</h3>"
         "<table>"
+        "<tr><td>Read representation"
+        "<td class=right>" << (assemblerInfo->readRepresentation==1 ? "1 (RLE)" : "0 (Raw - no RLE)") <<
         "<tr><td>Minimum read length"
         "<td class=right>" << assemblerInfo->minReadLength <<
         "<tr><td>Number of reads"
         "<td class=right>" << assemblerInfo->readCount <<
-        "<tr><td>Number of raw sequence bases"
+        "<tr><td>Number of read bases"
         "<td class=right>" << assemblerInfo->baseCount <<
-        "<tr><td>Average read length (for raw read sequence)"
+        "<tr><td>Average read length"
         "<td class=right>" << assemblerInfo->baseCount / assemblerInfo->readCount <<
-        "<tr><td>Read N50 (for raw read sequence)"
-        "<td class=right>" << assemblerInfo->readN50 <<
+        "<tr><td>Read N50"
+        "<td class=right>" << assemblerInfo->readN50;
+
+    if(assemblerInfo->readRepresentation == 1) {
+        html <<
         "<tr><td>Number of run-length encoded bases"
         "<td class=right>" << reads->getRepeatCountsTotalSize() <<
         "<tr><td>Average length ratio of run-length encoded sequence over raw sequence"
-        "<td class=right>" << setprecision(4) << double(reads->getRepeatCountsTotalSize()) / double(assemblerInfo->baseCount) <<
+        "<td class=right>" << setprecision(4) << double(reads->getRepeatCountsTotalSize()) / double(assemblerInfo->baseCount);
+    }
+
+    html <<
         "<tr><td>Number of reads flagged as palindromic by self alignment"
         "<td class=right>" << assemblerInfo->palindromicReadCount <<
         "<tr><td>Number of reads flagged as chimeric"
         "<td class=right>" << assemblerInfo->chimericReadCount <<
         "</table>"
         "<ul>"
-        "<li>Here and elsewhere, &quot;raw&quot; refers to the original read sequence, "
-        "as opposed to run-length encoded sequence."
         "<li>Reads discarded on input are not included in the above table (see "
         "<a href='#discarded'>below</a>)."
         "<li>See ReadLengthHistogram.csv and Binned-ReadLengthHistogram.csv "
@@ -923,9 +977,6 @@ void Assembler::writeAssemblySummaryBody(ostream& html, bool readsOnly)
         "<tr><td>Reads discarded on input because they contained repeat counts greater than 255"
         "<td class=right>" << assemblerInfo->discardedBadRepeatCountReadCount <<
         "<td class=right>" << assemblerInfo->discardedBadRepeatCountBaseCount <<
-        "<tr><td>Reads discarded on input because they had quality scores indicative of palindromic sequence"
-        "<td class=right>" << assemblerInfo->discardedPalindromicReadCount <<
-        "<td class=right>" << assemblerInfo->discardedPalindromicBaseCount <<
         "<tr><td>Reads discarded on input, total"
         "<td class=right>" <<totalDiscardedReadCount <<
         "<td class=right>" <<totalDiscardedBaseCount <<
@@ -943,9 +994,7 @@ void Assembler::writeAssemblySummaryBody(ostream& html, bool readsOnly)
         "<li>Here and elsewhere, &quot;raw&quot; refers to the original read sequence, "
         "as opposed to run-length encoded sequence.</ul>";
 
-    if (readsOnly) {
-        return;
-    }
+
 
     html <<
         "<h3>Marker <i>k</i>-mers</h3>"
@@ -969,24 +1018,36 @@ void Assembler::writeAssemblySummaryBody(ostream& html, bool readsOnly)
         "<tr><td>Total number of markers on all reads, one strand"
         "<td class=right>" << markers.totalSize()/2 <<
         "<tr><td>Total number of markers on all reads, both strands"
-        "<td class=right>" << markers.totalSize() <<
-        "<tr><td>Average number of markers per raw base"
-        "<td class=right>" << setprecision(4) << double(markers.totalSize()/2)/double(assemblerInfo->baseCount) <<
-        "<tr><td>Average number of markers per run-length encoded base"
-        "<td class=right>" << setprecision(4) << double(markers.totalSize()/2)/double(reads->getRepeatCountsTotalSize()) <<
-        "<tr><td>Average base offset between markers in raw sequence"
-        "<td class=right>" << setprecision(4) << double(assemblerInfo->baseCount)/double(markers.totalSize()/2) <<
-        "<tr><td>Average base offset between markers in run-length encoded sequence"
-        "<td class=right>" << setprecision(4) << double(reads->getRepeatCountsTotalSize())/double(markers.totalSize()/2) <<
-        "<tr><td>Average base gap between markers in run-length encoded sequence"
-        "<td class=right>" << setprecision(4) <<
-        double(reads->getRepeatCountsTotalSize())/double(markers.totalSize()/2) - double(assemblerInfo->k) <<
-        "</table>"
-        "<ul><li>Here and elsewhere, &quot;raw&quot; refers to the original read sequence, "
-        "as opposed to run-length encoded sequence.</ul>"
+        "<td class=right>" << markers.totalSize();
+
+    if(assemblerInfo->readRepresentation == 1) {
+        html <<
+            "<tr><td>Average number of markers per raw base"
+            "<td class=right>" << setprecision(4) << double(markers.totalSize()/2)/double(assemblerInfo->baseCount) <<
+            "<tr><td>Average number of markers per run-length encoded base"
+            "<td class=right>" << setprecision(4) << double(markers.totalSize()/2)/double(reads->getRepeatCountsTotalSize()) <<
+            "<tr><td>Average base offset between markers in raw sequence"
+            "<td class=right>" << setprecision(4) << double(assemblerInfo->baseCount)/double(markers.totalSize()/2) <<
+            "<tr><td>Average base offset between markers in run-length encoded sequence"
+            "<td class=right>" << setprecision(4) << double(reads->getRepeatCountsTotalSize())/double(markers.totalSize()/2) <<
+            "<tr><td>Average base gap between markers in run-length encoded sequence"
+            "<td class=right>" << setprecision(4) <<
+            double(reads->getRepeatCountsTotalSize())/double(markers.totalSize()/2) - double(assemblerInfo->k);
+    } else {
+        html <<
+            "<tr><td>Average number of markers per base"
+            "<td class=right>" << setprecision(4) << double(markers.totalSize()/2)/double(assemblerInfo->baseCount) <<
+            "<tr><td>Average base offset between markers "
+            "<td class=right>" << setprecision(4) << double(assemblerInfo->baseCount)/double(markers.totalSize()/2) <<
+            "<tr><td>Average base gap between markers"
+            "<td class=right>" << setprecision(4) <<
+            double(assemblerInfo->baseCount)/double(markers.totalSize()/2) - double(assemblerInfo->k);
+    }
+    html << "</table>";
 
 
 
+    html <<
         "<h3>Alignments</h3>"
         "<table>"
         "<tr><td>Number of alignment candidates found by the LowHash algorithm"
@@ -999,6 +1060,17 @@ void Assembler::writeAssemblySummaryBody(ostream& html, bool readsOnly)
 
 
 
+        "<h3>Alignment criteria actually used for creation of the read graph</h3>"
+        "<table>"
+        "<tr><td>minAlignedMarkerCount<td class=right>" << assemblerInfo->actualMinAlignedMarkerCount <<
+        "<tr><td>minAlignedFraction<td class=right>" << assemblerInfo->actualMinAlignedFraction <<
+        "<tr><td>maxSkip<td class=right>" << assemblerInfo->actualMaxSkip <<
+        "<tr><td>maxDrift<td class=right>" << assemblerInfo->actualMaxDrift <<
+        "<tr><td>maxTrim<td class=right>" << assemblerInfo->actualMaxTrim <<
+        "</table>"
+
+
+
         "<h3>Read graph</h3>"
         "<table>"
         "<tr><td>Number of vertices"
@@ -1060,27 +1132,75 @@ void Assembler::writeAssemblySummaryBody(ostream& html, bool readsOnly)
 
 
 
-        html <<
-        "<h3>Assembled segments (&quot;contigs&quot;)</h3>"
-        "<table>"
-        "<tr><td>Number of segments assembled"
-        "<td class=right>" << assemblerInfo->assemblyGraphAssembledEdgeCount <<
-        "<tr><td>Total assembled segment length"
-        "<td class=right>" << assemblerInfo->totalAssembledSegmentLength <<
-        "<tr><td>Longest assembled segment length"
-        "<td class=right>" << assemblerInfo->longestAssembledSegmentLength <<
-        "<tr><td>Assembled segments N<sub>50</sub>"
-        "<td class=right>" << assemblerInfo->assembledSegmentN50 <<
-        "</table>"
-        "<ul><li>Shasta uses GFA terminology "
-        "(<i>segment</i> instead of the most common <i>contig</i>). "
-        "A contiguous section of assembled sequence can consist of multiple segments, "
-        "for example in the presence of heterozygous bubbles."
-        "<li>See AssemblySummary.csv for lengths of assembled segments."
-        "</ul>"
+        if(assemblerInfo->assemblyMode == 0) {
+            html <<
+            "<h3>Assembled segments (&quot;contigs&quot;)</h3>"
+            "<table>"
+            "<tr><td>Number of segments assembled"
+            "<td class=right>" << assemblerInfo->assemblyGraphAssembledEdgeCount <<
+            "<tr><td>Total assembled segment length"
+            "<td class=right>" << assemblerInfo->totalAssembledSegmentLength <<
+            "<tr><td>Longest assembled segment length"
+            "<td class=right>" << assemblerInfo->longestAssembledSegmentLength <<
+            "<tr><td>Assembled segments N<sub>50</sub>"
+            "<td class=right>" << assemblerInfo->assembledSegmentN50 <<
+            "</table>"
+            "<ul><li>Shasta uses GFA terminology "
+            "(<i>segment</i> instead of the most common <i>contig</i>). "
+            "A contiguous section of assembled sequence can consist of multiple segments, "
+            "for example in the presence of heterozygous bubbles."
+            "<li>See AssemblySummary.csv for lengths of assembled segments."
+            "</ul>";
+        }
 
 
 
+        if(assemblerInfo->assemblyMode == 2) {
+            const AssemblyGraph2Statistics& statistics = assemblerInfo->assemblyGraph2Statistics;
+            html <<
+                "<h3>Phased assembly statistics</h3>"
+                "<table>"
+                "<tr><th><th>Length<th>N<sub>50</sub>"
+                "<tr><td>Bubble chains"
+                "<td class=right>" << statistics.totalBubbleChainLength <<
+                "<td class=right>" << statistics.bubbleChainN50 <<
+                "<tr><td>Diploid sequence (per haplotype)"
+                "<td class=right>" << statistics.totalDiploidLengthBothHaplotypes / 2 <<
+                "<td class=right>" << statistics.diploidN50 <<
+                "<tr><td>Haploid sequence"
+                "<td class=right>" << statistics.totalHaploidLength <<
+                "<td class=right>" << statistics.haploidN50 <<
+                "<tr><td>Total sequence assembled in bubble chains, per haplotype"
+                "<td class=right>" <<
+                statistics.totalDiploidLengthBothHaplotypes / 2 +
+                statistics.totalHaploidLength <<
+                "<td class=right>"
+                "<tr><td>Sequence outside bubble chains"
+                "<td class=right>" << statistics.outsideBubbleChainsLength <<
+                "<td class=right>"
+                "</table>"
+
+                "<p>"
+                "<table>"
+                "<tr><td>Number of bubbles that describe a single SNP (transition)"
+                "<td class=right>" << statistics.simpleSnpBubbleTransitionCount <<
+                "<tr><td>Number of bubbles that describe a single SNP (transversion)"
+                "<td class=right>" << statistics.simpleSnpBubbleTransversionCount <<
+                "<tr><td>Number of bubbles that describe a single SNP (total)"
+                "<td class=right>" <<
+                statistics.simpleSnpBubbleTransitionCount +
+                statistics.simpleSnpBubbleTransversionCount <<
+                "<tr><td>Transition/transversion ratio for bubbles that describe a single SNP"
+                "<td class=right>" <<
+                double(statistics.simpleSnpBubbleTransitionCount) /
+                double(statistics.simpleSnpBubbleTransversionCount) <<
+                "<tr><td>Number of bubbles that describe indels or more than one SNP"
+                "<td class=right>" << statistics.nonSimpleSnpBubbleCount <<
+                "</table>";
+        }
+
+
+    html <<
         "<h3>Performance</h3>"
         "<table>"
         "<tr><td>Elapsed time (seconds)"
@@ -1093,14 +1213,14 @@ void Assembler::writeAssemblySummaryBody(ostream& html, bool readsOnly)
         "<td class=right>" << assemblerInfo->averageCpuUtilization <<
         "<tr><td>Peak Memory utilization (bytes)"
         "<td class=right>" <<
-	assemblerInfo->peakMemoryUsageForSummaryStats() <<
+        assemblerInfo->peakMemoryUsageForSummaryStats() <<
         "</table>"
         ;
 }
 
 
 
-void Assembler::writeAssemblySummaryJson(ostream& json, bool readsOnly)
+void Assembler::writeAssemblySummaryJson(ostream& json)
 {
     AssemblyGraph& assemblyGraph = *assemblyGraphPointer;
     using std::setprecision;
@@ -1122,12 +1242,10 @@ void Assembler::writeAssemblySummaryJson(ostream& json, bool readsOnly)
     const uint64_t totalDiscardedReadCount =
         assemblerInfo->discardedInvalidBaseReadCount +
         assemblerInfo->discardedShortReadReadCount +
-        assemblerInfo->discardedPalindromicReadCount +
         assemblerInfo->discardedBadRepeatCountReadCount;
     const uint64_t totalDiscardedBaseCount =
         assemblerInfo->discardedInvalidBaseBaseCount +
         assemblerInfo->discardedShortReadBaseCount +
-        assemblerInfo->discardedPalindromicBaseCount +
         assemblerInfo->discardedBadRepeatCountBaseCount;
 
 
@@ -1143,15 +1261,23 @@ void Assembler::writeAssemblySummaryJson(ostream& json, bool readsOnly)
 
         "  \"Reads used in this assembly\":\n"
         "  {\n"
+        "    \"Read representation\": \"" << (assemblerInfo->readRepresentation==1 ? "1 (RLE)" : "0 (Raw - no RLE)") << "\",\n"
         "    \"Minimum read length\": " << assemblerInfo->minReadLength << ",\n"
         "    \"Number of reads\": " << assemblerInfo->readCount << ",\n"
-        "    \"Number of raw sequence bases\": " << assemblerInfo->baseCount << ",\n"
-        "    \"Average read length (for raw read sequence)\": " <<
+        "    \"Number of read bases\": " << assemblerInfo->baseCount << ",\n"
+        "    \"Average read length\": " <<
         assemblerInfo->baseCount / assemblerInfo->readCount << ",\n"
-        "    \"Read N50 (for raw read sequence)\": " << assemblerInfo->readN50 << ",\n"
-        "    \"Number of run-length encoded bases\": " << reads->getRepeatCountsTotalSize() << ",\n"
-        "    \"Average length ratio of run-length encoded sequence over raw sequence\": " <<
-        setprecision(4) << double(reads->getRepeatCountsTotalSize()) / double(assemblerInfo->baseCount) << ",\n"
+        "    \"Read N50\": " << assemblerInfo->readN50 << ",\n";
+
+    if(assemblerInfo->readRepresentation == 1) {
+        json <<
+            "    \"Number of run-length encoded bases\": " << reads->getRepeatCountsTotalSize() << ",\n"
+            "    \"Average length ratio of run-length encoded sequence over raw sequence\": " <<
+            setprecision(4) <<
+            double(reads->getRepeatCountsTotalSize()) / double(assemblerInfo->baseCount) << ",\n";
+    }
+
+    json <<
         "    \"Number of reads flagged as palindromic by self alignment\": " << assemblerInfo->palindromicReadCount << ",\n"
         "    \"Number of reads flagged as chimeric\": " << assemblerInfo->chimericReadCount << "\n"
         "  },\n"
@@ -1175,11 +1301,6 @@ void Assembler::writeAssemblySummaryJson(ostream& json, bool readsOnly)
         "      \"Reads\": " << assemblerInfo->discardedBadRepeatCountReadCount << ",\n"
         "      \"Bases\": " << assemblerInfo->discardedBadRepeatCountBaseCount << "\n"
         "    },\n"
-        "    \"Reads discarded on input because they they had quality scores indicative of palindromic sequence\":\n"
-        "    {\n"
-        "      \"Reads\": " << assemblerInfo->discardedPalindromicReadCount << ",\n"
-        "      \"Bases\": " << assemblerInfo->discardedPalindromicBaseCount << "\n"
-        "    },\n"
         "    \"Reads discarded on input, total\":\n"
         "    {\n"
         "      \"Reads\": " << totalDiscardedReadCount << ",\n"
@@ -1198,9 +1319,6 @@ void Assembler::writeAssemblySummaryJson(ostream& json, bool readsOnly)
         "    }\n"
         "  },\n";
 
-    if (readsOnly) {
-        return;
-    }
 
     json <<
         "  \"Marker k-mers\":\n"
@@ -1219,18 +1337,36 @@ void Assembler::writeAssemblySummaryJson(ostream& json, bool readsOnly)
         "    \"Total number of markers on all reads, one strand\": "
         << markers.totalSize()/2 << ",\n"
         "    \"Total number of markers on all reads, both strands\": "
-        << markers.totalSize() << ",\n"
-        "    \"Average number of markers per raw base\": "
-        << setprecision(4) << double(markers.totalSize()/2)/double(assemblerInfo->baseCount) << ",\n"
-        "    \"Average number of markers per run-length encoded base\": "
-        << setprecision(4) << double(markers.totalSize()/2)/double(reads->getRepeatCountsTotalSize()) << ",\n"
-        "    \"Average base offset between markers in raw sequence\": "
-        << setprecision(4) << double(assemblerInfo->baseCount)/double(markers.totalSize()/2) << ",\n"
-        "    \"Average base offset between markers in run-length encoded sequence\": "
-        << setprecision(4) << double(reads->getRepeatCountsTotalSize())/double(markers.totalSize()/2) << ",\n"
-        "    \"Average base gap between markers in run-length encoded sequence\": "
-        << setprecision(4) <<
-        double(reads->getRepeatCountsTotalSize())/double(markers.totalSize()/2) - double(assemblerInfo->k) << "\n"
+        << markers.totalSize() << ",\n";
+
+    if(assemblerInfo->readRepresentation == 1) {
+        json <<
+            "    \"Average number of markers per raw base\": "
+            << setprecision(4) << double(markers.totalSize()/2)/double(assemblerInfo->baseCount) << ",\n"
+            "    \"Average number of markers per run-length encoded base\": "
+            << setprecision(4) << double(markers.totalSize()/2)/double(reads->getRepeatCountsTotalSize()) << ",\n"
+            "    \"Average base offset between markers in raw sequence\": "
+            << setprecision(4) << double(assemblerInfo->baseCount)/double(markers.totalSize()/2) << ",\n"
+            "    \"Average base offset between markers in run-length encoded sequence\": "
+            << setprecision(4) << double(reads->getRepeatCountsTotalSize())/double(markers.totalSize()/2) << ",\n"
+            "    \"Average base gap between markers in run-length encoded sequence\": "
+            << setprecision(4) <<
+            double(reads->getRepeatCountsTotalSize())/double(markers.totalSize()/2) - double(assemblerInfo->k) << "\n";
+    } else {
+        json <<
+            "    \"Average number of markers per base\": "
+            << setprecision(4) << double(markers.totalSize()/2)/double(assemblerInfo->baseCount) << ",\n"
+            "    \"Average base offset between markers\": "
+            << setprecision(4) << double(assemblerInfo->baseCount)/double(markers.totalSize()/2) << ",\n"
+            "    \"Average base gap between markers\": "
+            << setprecision(4) <<
+            double(assemblerInfo->baseCount)/double(markers.totalSize()/2) - double(assemblerInfo->k) << "\n";
+
+    }
+
+
+
+    json <<
         "  },\n"
 
 
@@ -1244,6 +1380,17 @@ void Assembler::writeAssemblySummaryJson(ostream& json, bool readsOnly)
 
 
 
+        "  \"Alignment criteria actually used for creation of the read graph\":\n"
+        "  {\n"
+        "    \"minAlignedMarkerCount\": " << assemblerInfo->actualMinAlignedMarkerCount << ",\n"
+        "    \"minAlignedFraction\": " << assemblerInfo->actualMinAlignedFraction << ",\n"
+        "    \"maxSkip\": " << assemblerInfo->actualMaxSkip << ",\n"
+        "    \"maxDrift\": " << assemblerInfo->actualMaxDrift << ",\n"
+        "    \"maxTrim\": " << assemblerInfo->actualMaxTrim << "\n"
+        "  },\n"
+
+
+
         "  \"Read graph\":\n"
         "  {\n"
         "    \"Number of vertices\": " << readGraph.connectivity.size() << ",\n"
@@ -1293,17 +1440,66 @@ void Assembler::writeAssemblySummaryJson(ostream& json, bool readsOnly)
     }
 
 
+    if(assemblerInfo->assemblyMode == 0) {
+        json <<
+            "  \"Assembled segments\":\n"
+            "  {\n"
+            "    \"Number of segments assembled\": " << assemblerInfo->assemblyGraphAssembledEdgeCount << ",\n"
+            "    \"Total assembled segment length\": " << assemblerInfo->totalAssembledSegmentLength << ",\n"
+            "    \"Longest assembled segment length\": " << assemblerInfo->longestAssembledSegmentLength << ",\n"
+            "    \"Assembled segments N50\": " << assemblerInfo->assembledSegmentN50 << "\n"
+            "  },\n";
+    }
+
+
+
+    if(assemblerInfo->assemblyMode == 2) {
+        const AssemblyGraph2Statistics& statistics = assemblerInfo->assemblyGraph2Statistics;
+
+        json <<
+            "  \"Phased assembly statistics\":\n"
+            "  {\n"
+            "    \"Bubble chains\":\n"
+            "    {\n"
+            "      \"Length\": " << statistics.totalBubbleChainLength << ",\n"
+            "      \"N50\": " << statistics.bubbleChainN50 << "\n"
+            "    },\n"
+            "    \"Diploid sequence (per haplotype)\":\n"
+            "    {\n"
+            "      \"Length\": " << statistics.totalDiploidLengthBothHaplotypes / 2 << ",\n"
+            "      \"N50\": " << statistics.diploidN50 << "\n"
+            "    },\n"
+            "    \"Haploid sequence\":\n"
+            "    {\n"
+            "      \"Length\": " << statistics.totalHaploidLength << ",\n"
+            "      \"N50\": " << statistics.haploidN50 << "\n"
+            "    },\n"
+            "    \"Total sequence assembled in bubble chains, per haplotype\": " <<
+            statistics.totalDiploidLengthBothHaplotypes / 2 +
+            statistics.totalHaploidLength << ",\n"
+            "    \"Sequence outside bubble chains\": " <<
+            statistics.outsideBubbleChainsLength << ",\n"
+            "    \"Number of bubbles that describe a single SNP (transition)\": " <<
+            statistics.simpleSnpBubbleTransitionCount << ",\n"
+            "    \"Number of bubbles that describe a single SNP (transversion)\": " <<
+            statistics.simpleSnpBubbleTransversionCount << ",\n"
+            "    \"Number of bubbles that describe a single SNP (total)\": " <<
+            statistics.simpleSnpBubbleTransitionCount + statistics.simpleSnpBubbleTransversionCount << ",\n"
+            "    \"Transition/transversion ratio for bubbles that describe a single SNP\": " <<
+            (
+                // Make sure not to write a nan to json - it is not allowed.
+                (statistics.simpleSnpBubbleTransversionCount > 0) ?
+                (double(statistics.simpleSnpBubbleTransitionCount) / double(statistics.simpleSnpBubbleTransversionCount)) :
+                0.
+            ) << ",\n"
+            "    \"Number of bubbles that describe indels or more than one SNP\": " <<
+            statistics.nonSimpleSnpBubbleCount << "\n"
+            "  },\n";
+    }
+
+
+
     json <<
-        "  \"Assembled segments\":\n"
-        "  {\n"
-        "    \"Number of segments assembled\": " << assemblerInfo->assemblyGraphAssembledEdgeCount << ",\n"
-        "    \"Total assembled segment length\": " << assemblerInfo->totalAssembledSegmentLength << ",\n"
-        "    \"Longest assembled segment length\": " << assemblerInfo->longestAssembledSegmentLength << ",\n"
-        "    \"Assembled segments N50\": " << assemblerInfo->assembledSegmentN50 << "\n"
-        "  },\n"
-
-
-
         "  \"Performance\":\n"
         "  {\n"
         "    \"Elapsed time (seconds)\": " << assemblerInfo->assemblyElapsedTimeSeconds << ",\n"
@@ -1311,8 +1507,8 @@ void Assembler::writeAssemblySummaryJson(ostream& json, bool readsOnly)
         "    \"Elapsed time (hours)\": " << assemblerInfo->assemblyElapsedTimeSeconds/3600. << ",\n"
         "    \"Average CPU utilization\": " << assemblerInfo->averageCpuUtilization << ",\n"
         "    \"Peak Memory utilization (bytes)\": " <<
-	assemblerInfo->peakMemoryUsageForSummaryStats() <<
-	"\n"
+        assemblerInfo->peakMemoryUsageForSummaryStats() <<
+        "\n"
         "  }\n"
 
 
