@@ -36,6 +36,9 @@ void Assembler::randomlySelectKmers(
         throw runtime_error("K-mer capacity exceeded.");
     }
     assemblerInfo->k = k;
+
+    // The total number of k-mers of this length.
+    // This includes both RLE and non-RLE k-mers.
     const size_t kmerCount = 1ULL << (2ULL*k);
 
     // Sanity check on the requested fraction.
@@ -65,7 +68,7 @@ void Assembler::randomlySelectKmers(
     // So, probability = 1 - (1-p)^2, and therefore p=1-sqrt(1-probability).
     // For simplicity, we use the same p for k-mers that are
     // self-complementary. They are a small minority, and because
-    //of this they are chose with lower probability.
+    // of this they are chose with lower probability.
     // Probably a good thing anyway.
     const double p = 1. - sqrt(1. - probability);
     if(probability == 1.) {
@@ -88,22 +91,48 @@ void Assembler::randomlySelectKmers(
             kmerTable[reverseComplementedKmerId].isMarker = true;
         }
     }
-    size_t usedKmerCount = 0;
+
+
+    // Do some counting.
+    uint64_t rleKmerCount = 0;
+    uint64_t markerKmerCount = 0;
+    uint64_t markerRleKmerCount = 0;
     for(uint64_t kmerId=0; kmerId<kmerCount; kmerId++) {
-        if(kmerTable[kmerId].isMarker) {
-            ++usedKmerCount;
+        const KmerInfo& kmerInfo = kmerTable[kmerId];
+        if(kmerInfo.isRleKmer) {
+            ++rleKmerCount;
+        }
+        if(kmerInfo.isMarker) {
+            ++markerKmerCount;
+        }
+        if(kmerInfo.isRleKmer and kmerInfo.isMarker) {
+            ++markerRleKmerCount;
         }
     }
-    cout << "Selected " << usedKmerCount << " " << k << "-mers as markers out of ";
-    cout << kmerCount << " total." << endl;
-    cout << "Requested inclusion probability: " << probability << "." << endl;
-    cout << "Actual fraction of marker k-mers: ";
-    cout << double(usedKmerCount)/double(kmerCount) << "." << endl;
-    cout << "The above statistics include all k-mers, not just those present in "
-        "run-length encoded sequence." << endl;
 
-    if(probability == 1.) {
-        SHASTA_ASSERT(usedKmerCount == kmerCount);
+
+
+    // Summary messages.
+    if(assemblerInfo->readRepresentation == 0) {
+
+        // We are using the raw representation of the reads.
+        cout << "Total number of k-mers of length " << k << " is " << kmerCount << endl;
+        cout << "Of those, " << markerKmerCount << " will be used as markers." << endl;
+        cout << "Fraction of k-mers used as markers: requested " <<
+            probability << ", actual " <<
+            double(markerKmerCount)/double(kmerCount) << "." << endl;
+
+
+    } else {
+
+        // We are using the RLE representation of the reads.
+        cout << "Total number of k-mers of length " << k << " is " << kmerCount << endl;
+        cout << "Number of RLE k-mers of length " << k << " is " << rleKmerCount << endl;
+        cout << "Of those, " << markerRleKmerCount << " will be used as markers." << endl;
+        cout << "Fraction of k-mers used as markers: requested " <<
+            probability << ", actual " <<
+            double(markerRleKmerCount)/double(rleKmerCount) << "." << endl;
+
     }
 
 }
@@ -227,26 +256,52 @@ void Assembler::selectKmersBasedOnFrequency(
     setupLoadBalancing(reads->readCount(), 1000);
     runThreads(&Assembler::computeKmerFrequency, threadCount);
 
-    // Compute the total number of k-mer occurrences
-    // and the number of RLE kmers.
+    // Compute the total number of k-mer occurrences in reads
+    // and the number of k-mers that can possibly occur.
+    // This is done by counting all k-mers
+    // when using the raw read representation and
+    // only RLE k-mers when using the RLE read representation.
     uint64_t totalKmerOccurrences = 0;
-    uint64_t rleKmerCount = 0;
+    uint64_t possibleKmerCount = 0;
     for(uint64_t kmerId=0; kmerId!=kmerTable.size(); kmerId++) {
         const KmerInfo& info = kmerTable[kmerId];
         totalKmerOccurrences += info.frequency;
-        if(info.isRleKmer) {
-            ++rleKmerCount;
+        if(assemblerInfo->readRepresentation == 0) {
+            ++possibleKmerCount;
+        } else {
+            if(info.isRleKmer) {
+                ++possibleKmerCount;
+            }
         }
     }
     const double averageOccurrenceCount =
-        double(totalKmerOccurrences) / double(rleKmerCount);
-    cout <<
-        "K-mer length k " << k << "\n"
-        "Total number of k-mers " << kmerTable.size() << "\n"
-        "Total number of RLE k-mers " << rleKmerCount << "\n"
-        "Total number of k-mer occurrences " << totalKmerOccurrences << "\n"
-        "Average number of occurrences per RLE k-mer " <<
-        averageOccurrenceCount << endl;
+        double(totalKmerOccurrences) / double(possibleKmerCount);
+
+
+
+    if(assemblerInfo->readRepresentation == 0) {
+
+        // We are using raw read representation.
+        cout <<
+            "K-mer length k " << k << "\n"
+            "Total number of k-mers " << kmerTable.size() << "\n"
+            "Total number of k-mer occurrences in all oriented reads " << totalKmerOccurrences << "\n"
+            "Average number of occurrences per k-mer " <<
+            averageOccurrenceCount << endl;
+
+    } else {
+
+        // We are using RLE read representation.
+        cout <<
+            "K-mer length k " << k << "\n"
+            "Total number of k-mers " << kmerTable.size() << "\n"
+            "Total number of RLE k-mers " << possibleKmerCount << "\n"
+            "Total number of k-mer occurrences in all oriented reads " << totalKmerOccurrences << "\n"
+            "Average number of occurrences per RLE k-mer " <<
+            averageOccurrenceCount << endl;
+    }
+
+
 
     // Convert the enrichment threshold to a frequency.
     const uint64_t frequencyThreshold =
@@ -260,10 +315,6 @@ void Assembler::selectKmersBasedOnFrequency(
     for(uint64_t kmerId=0; kmerId<kmerTable.size(); kmerId++) {
         const KmerInfo& info = kmerTable[kmerId];
         const uint64_t frequency = info.frequency;
-        if(!info.isRleKmer) {
-            SHASTA_ASSERT(frequency == 0);
-            continue;
-        }
 
         const Kmer kmer(kmerId, k);
         const Kmer reverseComplementedKmer(info.reverseComplementedKmerId, k);
@@ -289,7 +340,7 @@ void Assembler::selectKmersBasedOnFrequency(
     vector<KmerId> candidateKmers;
     for(uint64_t kmerId=0; kmerId<kmerTable.size(); kmerId++) {
         const KmerInfo& info = kmerTable[kmerId];
-        if(!info.isRleKmer) {
+        if((assemblerInfo->readRepresentation==1) and  (not info.isRleKmer)) {
             continue;
         }
         const uint64_t frequency = info.frequency;
@@ -298,7 +349,7 @@ void Assembler::selectKmersBasedOnFrequency(
         }
         candidateKmers.push_back(KmerId(kmerId));
     }
-    cout << rleKmerCount - candidateKmers.size() << " k-mers were found to be "
+    cout << possibleKmerCount - candidateKmers.size() << " k-mers were found to be "
         "enriched by more than a factor of " << enrichmentThreshold <<
         " and will not be used as markers." << endl;
     cout << "Markers will be chosen randomly among the remaining pool of " <<
@@ -316,7 +367,7 @@ void Assembler::selectKmersBasedOnFrequency(
     }
 
 
-    // Now randomly generate markers fromthis pool of k-mers
+    // Now randomly generate markers from this pool of k-mers
     // until we have enough.
     uint64_t kmerOccurrencesCount = 0;
     uint64_t kmerCount = 0;
@@ -485,7 +536,7 @@ void Assembler::readKmersFromFile(uint64_t k, const string& fileName)
         const KmerId kmerId = KmerId(kmer.id(k));
         SHASTA_ASSERT(kmerId < kmerTable.size());
         KmerInfo& kmerInfo = kmerTable[kmerId];
-        if(not kmerInfo.isRleKmer) {
+        if((assemblerInfo->readRepresentation==1) and (not kmerInfo.isRleKmer)) {
             throw runtime_error("Non-RLE k-mer (duplicate consecutive bases) in " +
                 fileName + ":\n" + line);
         }
@@ -500,17 +551,21 @@ void Assembler::readKmersFromFile(uint64_t k, const string& fileName)
 
     // Count the number of k-mers flagged as markers.
     uint64_t usedKmerCount = 0;
-    uint64_t rleKmerCount = 0;
+    uint64_t possibleKmerCount = 0;
     for(const KmerInfo& kmerInfo: kmerTable) {
         if(kmerInfo.isMarker) {
             ++usedKmerCount;
         }
-        if(kmerInfo.isRleKmer) {
-            ++rleKmerCount;
+        if(assemblerInfo->readRepresentation == 0) {
+            ++possibleKmerCount;
+        } else {
+            if(kmerInfo.isRleKmer) {
+                ++possibleKmerCount;
+            }
         }
     }
-    cout << "Flagged as markers " << usedKmerCount << " out of " << rleKmerCount <<
-        " RLE k-mers of length " << k << endl;
+    cout << "Flagged as markers " << usedKmerCount << " out of " << possibleKmerCount <<
+        " possible k-mers of length " << k << endl;
 }
 
 
@@ -583,17 +638,21 @@ void Assembler::selectKmers2(
 
 
     // Compute the total number of k-mer occurrences
-    // and the number of RLE kmers.
+    // and the number of possible kmers.
     uint64_t totalKmerOccurrences = 0;
-    uint64_t rleKmerCount = 0;
+    uint64_t possibleKmerCount = 0;
     for(uint64_t kmerId=0; kmerId!=kmerTable.size(); kmerId++) {
         totalKmerOccurrences += selectKmers2Data.globalFrequency[kmerId];
-        if(kmerTable[kmerId].isRleKmer) {
-            ++rleKmerCount;
+        if(assemblerInfo->readRepresentation == 0) {
+            ++ possibleKmerCount;
+        } else {
+            if(kmerTable[kmerId].isRleKmer) {
+                ++possibleKmerCount;
+            }
         }
     }
     const double averageOccurrenceCount =
-        double(totalKmerOccurrences) / double(rleKmerCount);
+        double(totalKmerOccurrences) / double(possibleKmerCount);
 
 
 
@@ -604,10 +663,6 @@ void Assembler::selectKmers2(
     for(uint64_t kmerId=0; kmerId<kmerTable.size(); kmerId++) {
         const KmerInfo& info = kmerTable[kmerId];
         const uint64_t frequency = selectKmers2Data.globalFrequency[kmerId];
-        if(!info.isRleKmer) {
-            SHASTA_ASSERT(frequency == 0);
-            continue;
-        }
 
         const Kmer kmer(kmerId, k);
         const Kmer reverseComplementedKmer(info.reverseComplementedKmerId, k);
@@ -627,15 +682,15 @@ void Assembler::selectKmers2(
 
 
     // Gather k-mers that are not overenriched in any read and therefore
-    // can be used as markers..
+    // can be used as markers.
     vector<KmerId> candidateKmers;
     for(uint64_t kmerId=0; kmerId<kmerTable.size(); kmerId++) {
         if(kmerTable[kmerId].isRleKmer and selectKmers2Data.overenrichedReadCount[kmerId] == 0) {
             candidateKmers.push_back(KmerId(kmerId));
         }
     }
-    cout << "Out of a total " << rleKmerCount << " RLE k-mers, " <<
-        rleKmerCount - candidateKmers.size() <<
+    cout << "Out of a total " << possibleKmerCount << " possible k-mers, " <<
+        possibleKmerCount - candidateKmers.size() <<
         " were found to be over-enriched by more than a factor of " <<
         enrichmentThreshold <<
         " in at least one read and will not be used as markers." << endl;
@@ -643,8 +698,12 @@ void Assembler::selectKmers2(
         candidateKmers.size() << " k-mers." << endl;
     cout << "The enrichment threshold of " << enrichmentThreshold <<
         " corresponds to one occurrence every " <<
-        double(rleKmerCount) / enrichmentThreshold <<
-        " RLE bases." << endl;
+        double(possibleKmerCount) / enrichmentThreshold <<
+        " bases";
+    if(assemblerInfo->readRepresentation == 1) {
+        cout << " (in RLE representation)";
+    }
+    cout << "." << endl;
 
 
     // Prepare to generate a random index into this vector of candidate k-mers.
@@ -729,12 +788,16 @@ void Assembler::selectKmers2ThreadFunction(size_t threadId)
     // Access the enrichmentThreshold.
     const double enrichmentThreshold = selectKmers2Data.enrichmentThreshold;
 
-    // Compute the total number of RLE k-mers.
+    // Compute the total number of possible k-mers.
     // It is needed below for overenrichment computations.
-    uint64_t rleKmerCount = 0;
+    uint64_t possibleKmerCount = 0;
     for(const KmerInfo& kmerInfo: kmerTable) {
-        if(kmerInfo.isRleKmer) {
-            ++rleKmerCount;
+        if(assemblerInfo->readRepresentation == 0) {
+            ++possibleKmerCount;
+        } else {
+            if(kmerInfo.isRleKmer) {
+                ++possibleKmerCount;
+            }
         }
     }
 
@@ -790,7 +853,7 @@ void Assembler::selectKmers2ThreadFunction(size_t threadId)
             // in this read.
             const uint32_t readKmerCount = uint32_t(read.baseCount + 1 - k);
             const uint32_t frequencyThreshold =
-                uint32_t(enrichmentThreshold * double(readKmerCount) / double(rleKmerCount));
+                uint32_t(enrichmentThreshold * double(readKmerCount) / double(possibleKmerCount));
 
             // See if any k-mers are over-enriched in this read.
             SHASTA_ASSERT(readKmerIds.size() == readKmerIdFrequencies.size());
