@@ -1,8 +1,17 @@
 #include "Assembler.hpp"
 #include "deduplicate.hpp"
+#include "orderPairs.hpp"
+#include "performanceLog.hpp"
 #include "Reads.hpp"
 using namespace shasta;
 
+// Seqan
+#include <seqan/align.h>
+
+// Boost libraries.
+#include <boost/pending/disjoint_sets.hpp>
+
+
 
 // "Strict" version of createMarkerGraphEdges.
 // Differences from createMarkerGraphEdges:
@@ -23,7 +32,7 @@ void Assembler::createMarkerGraphEdgesStrict(
     uint64_t minEdgeCoveragePerStrand,
     size_t threadCount)
 {
-    cout << timestamp << "createMarkerGraphEdgesStrict begins." << endl;
+    performanceLog << timestamp << "createMarkerGraphEdgesStrict begins." << endl;
 
     // Check that we have what we need.
     checkMarkersAreOpen();
@@ -95,7 +104,7 @@ void Assembler::createMarkerGraphEdgesStrict(
     markerGraph.edgeMarkerIntervals.unreserve();
 
     SHASTA_ASSERT(markerGraph.edges.size() == markerGraph.edgeMarkerIntervals.size());
-    cout << timestamp << "Found " << markerGraph.edges.size();
+    cout << "Found " << markerGraph.edges.size();
     cout << " edges for " << markerGraph.vertexCount() << " vertices." << endl;
 
 
@@ -103,7 +112,7 @@ void Assembler::createMarkerGraphEdgesStrict(
     // Now we need to create edgesBySource and edgesByTarget.
     createMarkerGraphEdgesBySourceAndTarget(threadCount);
 
-    cout << timestamp << "createMarkerGraphEdgesStrict ends." << endl;
+    performanceLog << timestamp << "createMarkerGraphEdgesStrict ends." << endl;
 }
 
 
@@ -449,7 +458,7 @@ void Assembler::createMarkerGraphSecondaryEdges(
 
 
     const VertexId vertexCount = markerGraph.vertexCount();
-    cout << timestamp << "createMarkerGraphSecondaryEdges begins." << endl;
+    performanceLog << timestamp << "createMarkerGraphSecondaryEdges begins." << endl;
     cout << "The initial marker graph has " << vertexCount <<
         " vertices and " << markerGraph.edges.size() << " edges." << endl;
 
@@ -599,3 +608,344 @@ void Assembler::createMarkerGraphSecondaryEdges(
 }
 
 
+
+// Cluster the oriented reads on a marker graph edge based on their sequence.
+// This returns a vector of connected components.
+// Each connected component is an index into the marker intervals for the edge.
+vector< vector<uint64_t> > Assembler::clusterMarkerGraphEdgeOrientedReads(
+    MarkerGraphEdgeId edgeId,
+    double errorRateThreshold,
+    bool debug) const
+{
+    // Seqan types and functions used below.
+    using TSequence = seqan::String<char>;
+    using TAlign = seqan::Align<TSequence, seqan::ArrayGaps>;
+    using seqan::assignSource;
+    using seqan::globalAlignment;
+    using seqan::row;
+    using seqan::Score;
+    using seqan::Simple;
+
+    // The length of each marker sequence.
+    const size_t k = assemblerInfo->k;
+
+    const MarkerGraph::Edge& edge = markerGraph.edges[edgeId];
+    const span<const MarkerInterval> markerIntervals = markerGraph.edgeMarkerIntervals[edgeId];
+    const uint64_t n = markerIntervals.size();
+    SHASTA_ASSERT(edge.coverage == n);
+
+
+
+    // Get the sequence for each oriented read.
+    // This includes sequence for the flanking markers.
+    vector<TSequence> sequences;
+    for(const MarkerInterval& markerInterval: markerIntervals) {
+        const OrientedReadId orientedReadId = markerInterval.orientedReadId;
+        const auto orientedReadMarkers = markers[orientedReadId.getValue()];
+
+        // Get the two markers.
+        const CompressedMarker& marker0 = orientedReadMarkers[markerInterval.ordinals[0]];
+        const CompressedMarker& marker1 = orientedReadMarkers[markerInterval.ordinals[1]];
+
+        // Get the position range, including the flanking markers.
+        const uint32_t positionBegin = marker0.position;
+        const uint32_t positionEnd = marker1.position + uint32_t(k);
+
+        // Get the sequence for this oriented read.
+        TSequence sequence;
+        for(uint32_t position=positionBegin; position!=positionEnd; ++position) {
+
+            if(assemblerInfo->readRepresentation == 1) {
+                Base base;
+                uint8_t repeatCount;
+                tie(base, repeatCount) = reads->getOrientedReadBaseAndRepeatCount(orientedReadId, position);
+                const char c = base.character();
+                for(uint64_t j=0; j<repeatCount; j++) {
+                    appendValue(sequence, c);
+                }
+            } else {
+                const Base base = reads->getOrientedReadBase(orientedReadId, position);
+                const char c = base.character();
+                appendValue(sequence, c);
+            }
+        }
+        sequences.push_back(sequence);
+    }
+
+
+    ofstream csv;
+    ofstream dot;
+    if(debug) {
+        csv.open("ClusterOrientedReads.csv");
+        dot.open("ClusterOrientedReads.dot");
+        dot << "graph G{\n";
+        for(uint64_t i=0; i<n; i++) {
+            dot << "\"" << markerIntervals[i].orientedReadId << "\";\n";
+        }
+    }
+
+
+    // Implicitly create an undirected graph with a vertex
+    // for each oriented read in this marker graph edge.
+    // If the edit distance between the sequences of two
+    // oriented reads is sufficiently small, add an edge
+    // betwene the corresponding vertices.
+    // Compute connected components of this undirected graph.
+    // each connected component corresponds to a cluster.
+
+    // Initialize incremental connected components.
+    vector<uint64_t> rank(n);
+    vector<uint64_t> parent(n);
+    boost::disjoint_sets<uint64_t*, uint64_t*> disjointSets(&rank[0], &parent[0]);
+    for(uint64_t i=0; i<n; i++) {
+        disjointSets.make_set(i);
+    }
+
+    // Loop over pairs of oriented reads.
+    for(uint64_t i0=0; i0<n-1; i0++) {
+        const TSequence& sequence0 = sequences[i0];
+
+        for(uint64_t i1=i0+1; i1<n; i1++) {
+            const TSequence& sequence1 = sequences[i1];
+
+            // Compute the error rate.
+            TAlign align;
+            resize(rows(align), 2);
+            assignSource(row(align,0), sequence0);
+            assignSource(row(align,1), sequence1);
+            const int editDistance = -globalAlignment(align, Score<int, Simple>(0,-1,-1));
+            const double errorRate = double(editDistance) / double(min(length(sequence0), length(sequence1)));
+
+            // If the error rate is small, merge the connected components
+            // corresponding to these two oriented reads.
+            if(errorRate < errorRateThreshold) {
+                disjointSets.union_set(i0, i1);
+            }
+
+            if(debug) {
+                const OrientedReadId orientedReadId0 = markerIntervals[i0].orientedReadId;
+                const OrientedReadId orientedReadId1 = markerIntervals[i1].orientedReadId;
+                csv << orientedReadId0 << ",";
+                csv << orientedReadId1 << ",";
+                csv << errorRate << "\n";
+                if((errorRate < errorRateThreshold)) {
+                    dot << "\"" << orientedReadId0 << "\"--\"" << orientedReadId1 << "\";\n";
+                }
+            }
+        }
+    }
+
+    if(debug) {
+        dot << "}\n";
+    }
+
+
+    // Gather connected components.
+    vector< vector<uint64_t> > components(n);
+    for(uint64_t i=0; i<n; i++) {
+        const uint64_t componentId = disjointSets.find_set(i);
+        components[componentId].push_back(i);
+    }
+
+    // Gather a vector of pairs (componentId, size) for non-empty components.
+    vector< pair<uint64_t, uint64_t> > componentTable;
+    for(uint64_t componentId=0; componentId<n; componentId++) {
+        const uint64_t componentSize = components[componentId].size();
+        if(componentSize > 0) {
+            componentTable.push_back(make_pair(componentId, componentSize));
+        }
+    }
+
+    // Sort it by decreasing component size.
+    sort(componentTable.begin(), componentTable.end(),
+        OrderPairsBySecondOnlyGreater<uint64_t, uint64_t>());
+
+    // Gather the connected components in order of drcreasing size.
+    vector< vector<uint64_t> > sortedComponents;
+    for(const auto& p: componentTable) {
+        sortedComponents.push_back(components[p.first]);
+    }
+
+    return sortedComponents;
+}
+
+
+
+// Use clusterMarkerGraphEdgeOrientedReads to split secondary marker graph edges
+// where necessary.
+void Assembler::splitMarkerGraphSecondaryEdges(
+    double errorRateThreshold,
+    uint64_t minCoverage,
+    size_t threadCount)
+{
+
+    // Adjust the numbers of threads, if necessary.
+    if(threadCount == 0) {
+        threadCount = std::thread::hardware_concurrency();
+    }
+
+    // Store parameters to all threads can see them.
+    auto& data = splitMarkerGraphSecondaryEdgesData;
+    data.errorRateThreshold = errorRateThreshold;
+    data.minCoverage = minCoverage;
+
+    // Initialize some counts and data structures.
+    data.initialSecondaryCount = 0;
+    data.splitCount = 0;
+    data.createdCount = 0;
+    data.threadEdges.resize(threadCount);
+
+    cout << "Total number of marker graph edges before splitting of secondary edge " <<
+        markerGraph.edges.size() << endl;
+
+    // The edge splitting is done in multithreaded code.
+    const uint64_t batchSize = 1000;
+    setupLoadBalancing(markerGraph.edges.size(), batchSize);
+    runThreads(&Assembler::splitMarkerGraphSecondaryEdgesThreadFunction, threadCount);
+
+    // Add the new edges to the marker graph.
+    for(size_t threadId=0; threadId<threadCount; threadId++) {
+        for(const auto& tmpNewEdge: data.threadEdges[threadId]) {
+            markerGraph.edges.resize(markerGraph.edges.size() + 1);
+            auto& newEdge = markerGraph.edges.back();
+            newEdge.source = Uint40(tmpNewEdge.source);
+            newEdge.target = Uint40(tmpNewEdge.target);
+            newEdge.coverage = uint8_t(tmpNewEdge.markerIntervals.size());
+            newEdge.isSecondary = 1;
+            markerGraph.edgeMarkerIntervals.appendVector(tmpNewEdge.markerIntervals);
+        }
+    }
+    data.threadEdges.clear();
+    data.threadEdges.shrink_to_fit();
+
+    cout << "Total number of marker graph edges after splitting of secondary edges " <<
+        markerGraph.edges.size() << endl;
+    cout << "Initial number of secondary edges " << data.initialSecondaryCount << endl;
+    cout << "Number of secondary edges that were split " << data.splitCount << endl;
+    cout << "Number of secondary edges that were created " << data.createdCount << endl;
+
+
+    // We need to recreate edgesBySource and edgesByTarget.
+    if(markerGraph.edgesBySource.isOpen()) {
+        markerGraph.edgesBySource.close();
+    }
+    if(markerGraph.edgesByTarget.isOpen()) {
+        markerGraph.edgesByTarget.close();
+    }
+    createMarkerGraphEdgesBySourceAndTarget(threadCount);
+
+    // We also need to recompute reverse complement marker graph edges.
+    if(markerGraph.reverseComplementEdge.isOpen) {
+        markerGraph.reverseComplementEdge.close();
+    }
+    findMarkerGraphReverseComplementEdges(threadCount);
+
+}
+
+
+
+void Assembler::splitMarkerGraphSecondaryEdgesThreadFunction(size_t threadId)
+{
+
+    auto& data = splitMarkerGraphSecondaryEdgesData;
+    const double errorRateThreshold = data.errorRateThreshold;
+    const uint64_t minCoverage = data.minCoverage;
+    auto& newEdges = data.threadEdges[threadId];
+
+    uint64_t initialSecondaryCount = 0;
+    uint64_t splitCount = 0;
+    uint64_t createdCount = 0;
+
+    // Loop over batches assigned to this thread.
+    uint64_t begin, end;
+    while(getNextBatch(begin, end)) {
+
+        // Loop over marker graph edges assigned to this batch
+        for(MarkerGraphEdgeId edgeId=begin; edgeId!=end; edgeId++) {
+            MarkerGraph::Edge& edge = markerGraph.edges[edgeId];
+
+            // Find the reverse complement edge.
+            const MarkerGraphEdgeId edgeIdRc = markerGraph.reverseComplementEdge[edgeId];
+
+            // Process each pair of reverse complemented edges only once.
+            SHASTA_ASSERT(edgeIdRc != edgeId);
+            if(edgeIdRc < edgeId) {
+                continue;
+            }
+            MarkerGraph::Edge& edgeRc = markerGraph.edges[edgeIdRc];
+
+            // Mark both of them as not removed due to splitting.
+            edge.wasRemovedWhileSplittingSecondaryEdges = 0;
+            edgeRc.wasRemovedWhileSplittingSecondaryEdges = 0;
+
+            // If these are not secondary edges, do nothing.
+            SHASTA_ASSERT(edge.isSecondary == edgeRc.isSecondary);
+            if(not edge.isSecondary) {
+                continue;
+            }
+            initialSecondaryCount += 2;
+
+            // Sanity check on the marker intervals.
+            const span<MarkerInterval> markerIntervals = markerGraph.edgeMarkerIntervals[edgeId];
+            const span<MarkerInterval> markerIntervalsRc = markerGraph.edgeMarkerIntervals[edgeIdRc];
+            SHASTA_ASSERT(markerIntervals.size() == markerIntervalsRc.size());
+            for(uint64_t i=0; i<markerIntervals.size(); i++) {
+                const OrientedReadId& orientedReadId = markerIntervals[i].orientedReadId;
+                const OrientedReadId& orientedReadIdRc = markerIntervalsRc[i].orientedReadId;
+                SHASTA_ASSERT(orientedReadId.getReadId() == orientedReadIdRc.getReadId());
+                SHASTA_ASSERT(orientedReadId.getStrand() == 1 - orientedReadIdRc.getStrand());
+            }
+
+            // Cluster oriented reads on this edge based on their sequence.
+            // The clusters are returned sorted by decreasing size.
+            const vector< vector<uint64_t> > clusters =
+                clusterMarkerGraphEdgeOrientedReads(edgeId, errorRateThreshold, false);
+
+            // If we only found one cluster, we don't need to do anything.
+            if(clusters.size() == 1) {
+                continue;
+            }
+
+            // We will split this edge and its reverse complement.
+            splitCount += 2;
+            edge.wasRemovedWhileSplittingSecondaryEdges = 1;
+            edgeRc.wasRemovedWhileSplittingSecondaryEdges = 1;
+
+            // Each cluster generates a new edge,
+            // if it is big enough. But always generate at least one.
+            for(uint64_t i=0; i<clusters.size(); i++) {
+                const vector<uint64_t>& cluster = clusters[i];
+
+                // If this cluster is too small and it is not the first
+                // cluster, stop here.
+                if((cluster.size() < minCoverage) and (i > 0)) {
+                    break;
+                }
+
+                // If getting here, generate a new edge for
+                // each of edge and edgeRc.
+                createdCount += 2;
+                newEdges.resize(newEdges.size() + 2);
+                auto& newEdge = newEdges[newEdges.size() - 2];
+                auto& newEdgeRc = newEdges[newEdges.size() - 1];
+
+                // Fill in the vertices.
+                newEdge.source = edge.source;
+                newEdge.target = edge.target;
+                newEdgeRc.source = edgeRc.source;
+                newEdgeRc.target = edgeRc.target;
+
+                // Fill in the marker intervals.
+                for(const uint64_t j: cluster) {
+                    newEdge.markerIntervals.push_back(markerIntervals[j]);
+                    newEdgeRc.markerIntervals.push_back(markerIntervalsRc[j]);
+                }
+            }
+        }
+    }
+
+    // Increment global counts.
+    __sync_fetch_and_add(&data.initialSecondaryCount, initialSecondaryCount);
+    __sync_fetch_and_add(&data.splitCount, splitCount);
+    __sync_fetch_and_add(&data.createdCount, createdCount);
+}
