@@ -188,7 +188,11 @@ void AssemblerOptions::addCommandLineOnlyOptions()
         default_value(0),
         "Number of threads, or 0 to use one thread per virtual processor.")
         
-#ifdef SHASTA_HTTP_SERVER
+        ("suppressStdoutLog",
+        bool_switch(&commandLineOnlyOptions.suppressStdoutLog)->
+        default_value(false),
+        "Suppress echoing stdout to stdout.log.")
+
         ("exploreAccess",
         value<string>(&commandLineOnlyOptions.exploreAccess)->
         default_value("user"),
@@ -209,7 +213,6 @@ void AssemblerOptions::addCommandLineOnlyOptions()
         "a reference. Only used for --command explore, for display of the alignment "
         "candidate graph. Experimental."
         )
-#endif
         ;
 
 
@@ -238,6 +241,12 @@ void AssemblerOptions::addConfigurableOptions()
 
     configurableOptionsDescription.add_options()
 
+        ("Reads.representation",
+        value<uint64_t>(&readsOptions.representation)->
+        default_value(1),
+        "Read representation: 0 = raw sequence, 1 (default) = Run-Length Encoded (RLE) sequence. "
+        "Experimental. Do not use.")
+
         ("Reads.minReadLength",
         value<int>(&readsOptions.minReadLength)->
         default_value(10000),
@@ -295,30 +304,6 @@ void AssemblerOptions::addConfigurableOptions()
          default_value(100),
          "Used for palindromic read detection.")
 
-        ("Reads.palindromicReads.detectOnFastqLoad",
-         bool_switch(&readsOptions.palindromicReads.detectOnFastqLoad)->
-         default_value(false),
-        "Filter reads that have exceptionally poor quality in the second half, "
-        "which is a strong indication of palindromic sequence.")
-
-        ("Reads.palindromicReads.qScoreRelativeMeanDifference",
-        value<double>(&readsOptions.palindromicReads.qScoreRelativeMeanDifference)->
-        default_value(0.09, "0.09"),
-        "When filtering palindrome quality, this parameter describes how much "
-        "worse should the right side average p(error) should be.")
-
-        ("Reads.palindromicReads.qScoreMinimumMean",
-        value<double>(&readsOptions.palindromicReads.qScoreMinimumMean)->
-        default_value(0.15, "0.15"),
-        "When filtering palindrome quality, this parameter describes the absolute "
-        "minimum average p(error) in the right half.")
-
-        ("Reads.palindromicReads.qScoreMinimumVariance",
-         value<double>(&readsOptions.palindromicReads.qScoreMinimumVariance)->
-         default_value(0.025, "0.025"),
-         "When filtering palindrome quality, this parameter describes the absolute "
-         "minimum variance in the right half.")
-
          ("Kmers.generationMethod",
          value<int>(&kmersOptions.generationMethod)->
          default_value(0),
@@ -699,12 +684,6 @@ void AssemblerOptions::addConfigurableOptions()
         "corresponding marker graph edges. A cross edge is defined as an edge v0->v1 "
         "with out-degree(v0)>1, in-degree(v1)>1.")
 
-        ("MarkerGraph.refineThreshold",
-        value<uint64_t>(&markerGraphOptions.refineThreshold)->
-        default_value(0),
-        "Experimental. Length threshold, in markers, for the marker graph refinement step, "
-        "or 0 to turn off the refinement step.")
-
         ("MarkerGraph.reverseTransitiveReduction",
         bool_switch(&markerGraphOptions.reverseTransitiveReduction)->
         default_value(false),
@@ -722,6 +701,21 @@ void AssemblerOptions::addConfigurableOptions()
         "Used in the automatic selection of --MarkerGraph.minCoverage when "
         "--MarkerGraph.minCoverage is set to 0.")
 
+        ("MarkerGraph.secondaryEdges.maxSkip",
+        value<uint64_t>(&markerGraphOptions.secondaryEdgesMaxSkip)->
+        default_value(1000000),
+        "Maximum number of markers skipped by a secondary edge (mode 2 assembly only).")
+
+        ("MarkerGraph.secondaryEdges.split.errorRateThreshold",
+        value<double>(&markerGraphOptions.secondaryEdgesSplitErrorRateThreshold)->
+        default_value(0.25),
+        "Error rate threshold used for splitting secondary edges (mode 2 assembly only).")
+
+        ("MarkerGraph.secondaryEdges.split.minCoverage",
+        value<uint64_t>(&markerGraphOptions.secondaryEdgesSplitMinCoverage)->
+        default_value(4),
+        "Minimum coverage for secondary edges generated during splitting (mode 2 assembly only).")
+
         ("Assembly.mode",
         value<uint64_t>(&assemblyOptions.mode)->
         default_value(0),
@@ -774,7 +768,7 @@ void AssemblerOptions::addConfigurableOptions()
         default_value(0),
         "Specify the method used to detangle the assembly graph. "
         "0 = no detangling, 1 = strict detangling, "
-        "2 = less strict detangling, controlled by Assembly.detangle.* options (experimental).")
+        "2 = less strict detangling, controlled by Assembly.detangle.* options.")
 
         ("Assembly.detangle.diagonalReadCountMin",
         value<uint64_t>(&assemblyOptions.detangleDiagonalReadCountMin)->
@@ -844,54 +838,106 @@ void AssemblerOptions::addConfigurableOptions()
         default_value(2),
         "Maximum distance for read graph bridge removal for iterative assembly (experimental).")
 
-        ("Assembly.bubbleRemoval.discordantRatioThreshold",
-        value<double>(&assemblyOptions.bubbleRemovalDiscordantRatioThreshold)->
-        default_value(0.2),
-        "Discordant ratio threshold for bubble removal (assembly mode 2 only, experimental).")
+        ("Assembly.mode2.strongBranchThreshold",
+        value<uint64_t>(&assemblyOptions.mode2Options.strongBranchThreshold)->
+        default_value(2),
+        "Minimum number of supporting reads required for a strong branch. Only used in Mode 2 assembly.")
 
-        ("Assembly.bubbleRemoval.ambiguityThreshold",
-        value<double>(&assemblyOptions.bubbleRemovalAmbiguityThreshold)->
-        default_value(0.5),
-        "Ambiguity threshold for bubble removal (assembly mode 2 only, experimental).")
+        ("Assembly.mode2.epsilon",
+        value<double>(&assemblyOptions.mode2Options.epsilon)->
+        default_value(0.1),
+        "Epsilon for the Bayesian model used for phasing and for bubble removal."
+        "This is the probability that a read appears on the wrong branch of a diploid bubble. "
+        "Only used in Mode 2 assembly.")
 
-        ("Assembly.bubbleRemoval.maxPeriod",
-        value<uint64_t>(&assemblyOptions.bubbleRemovalMaxPeriod)->
-        default_value(4),
-        "Maximum repeat period for bubble removal (assembly mode 2 only, experimental).")
-
-        ("Assembly.superbubbleRemoval.edgeLengthThreshold",
-        value<uint64_t>(&assemblyOptions.superbubbleRemovalEdgeLengthThreshold)->
-        default_value(6),
-        "Edge length threshold in markers for superbubble removal "
-        "(assembly mode 2 only, experimental).")
-
-        ("Assembly.phasing.minReadCount",
-        value<uint64_t>(&assemblyOptions.phasingMinReadCount)->
+        ("Assembly.mode2.bubbleRemoval.minConcordantReadCount",
+        value<uint64_t>(&assemblyOptions.mode2Options.minConcordantReadCountForBubbleRemoval)->
         default_value(3),
-        "Minimum number of reads for phasing (assembly mode 2 only, experimental).")
+        "Minimum number of concordant reads for bubble removal. "
+        "Only used in Mode 2 assembly.")
 
-        ("Assembly.suppressGfaOutput",
-        bool_switch(&assemblyOptions.suppressGfaOutput)->
+        ("Assembly.mode2.bubbleRemoval.maxDiscordantReadCount",
+        value<uint64_t>(&assemblyOptions.mode2Options.maxDiscordantReadCountForBubbleRemoval)->
+        default_value(6),
+        "Maximum number of discordant reads for bubble removal. "
+        "Only used in Mode 2 assembly.")
+
+        ("Assembly.mode2.bubbleRemoval.minLogP",
+        value<double>(&assemblyOptions.mode2Options.minLogPForBubbleRemoval)->
+        default_value(30.),
+        "Minimul log(P) (in decibels) for bubble removal. "
+        "Only used in Mode 2 assembly.")
+
+        ("Assembly.mode2.bubbleRemoval.componentSizeThreshold",
+        value<uint64_t>(&assemblyOptions.mode2Options.componentSizeThresholdForBubbleRemoval)->
+        default_value(10),
+        "Component size threshold for bubble removal. "
+        "Only used in Mode 2 assembly.")
+
+        ("Assembly.mode2.phasing.minConcordantReadCount",
+        value<uint64_t>(&assemblyOptions.mode2Options.minConcordantReadCountForPhasing)->
+        default_value(2),
+        "Minimum number of concordant reads for phasing. "
+        "Only used in Mode 2 assembly.")
+
+        ("Assembly.mode2.phasing.maxDiscordantReadCount",
+        value<uint64_t>(&assemblyOptions.mode2Options.maxDiscordantReadCountForPhasing)->
+        default_value(1),
+        "Maximum number of discordant reads for phasing. "
+        "Only used in Mode 2 assembly.")
+
+        ("Assembly.mode2.phasing.minLogP",
+        value<double>(&assemblyOptions.mode2Options.minLogPForPhasing)->
+        default_value(10.),
+        "Minimul log(P) (in decibels) for phasing. "
+        "Only used in Mode 2 assembly.")
+
+        ("Assembly.mode2.superbubble.maxSize",
+        value<uint64_t>(&assemblyOptions.mode2Options.maxSuperbubbleSize)->
+        default_value(50),
+        "Maximum size (number of edges) of a superbubble to be processed. "
+        "Only used in Mode 2 assembly.")
+
+        ("Assembly.mode2.superbubble.maxChunkSize",
+        value<uint64_t>(&assemblyOptions.mode2Options.maxSuperbubbleChunkSize)->
+        default_value(20),
+        "Maximum size (numbef of edges) of a superbubble chunk to be processed. "
+        "Only used in Mode 2 assembly.")
+
+        ("Assembly.mode2.superbubble.maxChunkPathCount",
+        value<uint64_t>(&assemblyOptions.mode2Options.maxSuperbubbleChunkPathCount)->
+        default_value(20),
+        "Maximum number of paths to be processed in a superbubble chunk. "
+        "Only used in Mode 2 assembly.")
+
+        ("Assembly.mode2.superbubble.edgeLengthThreshold",
+        value<uint64_t>(&assemblyOptions.mode2Options.superbubbleEdgeLengthThreshold)->
+        default_value(6),
+        "Edge length threshold (in markers) for superbubble removal. "
+        "Only used in Mode 2 assembly.")
+
+        ("Assembly.mode2.suppressGfaOutput",
+        bool_switch(&assemblyOptions.mode2Options.suppressGfaOutput)->
         default_value(false),
         "Suppress all GFA output (Mode 2 assembly only).")
 
-        ("Assembly.suppressFastaOutput",
-        bool_switch(&assemblyOptions.suppressFastaOutput)->
+        ("Assembly.mode2.suppressFastaOutput",
+        bool_switch(&assemblyOptions.mode2Options.suppressFastaOutput)->
         default_value(false),
         "Suppress all FASTA output (Mode 2 assembly only).")
 
-        ("Assembly.suppressDetailedOutput",
-        bool_switch(&assemblyOptions.suppressDetailedOutput)->
+        ("Assembly.mode2.suppressDetailedOutput",
+        bool_switch(&assemblyOptions.mode2Options.suppressDetailedOutput)->
         default_value(false),
         "Suppress output of detailed representation of the assembly (Mode 2 assembly only).")
 
-        ("Assembly.suppressPhasedOutput",
-        bool_switch(&assemblyOptions.suppressPhasedOutput)->
+        ("Assembly.mode2.suppressPhasedOutput",
+        bool_switch(&assemblyOptions.mode2Options.suppressPhasedOutput)->
         default_value(false),
         "Suppress output of phased representation of the assembly (Mode 2 assembly only).")
 
-        ("Assembly.suppressHaploidOutput",
-        bool_switch(&assemblyOptions.suppressHaploidOutput)->
+        ("Assembly.mode2.suppressHaploidOutput",
+        bool_switch(&assemblyOptions.mode2Options.suppressHaploidOutput)->
         default_value(false),
         "Suppress output of haploid representation of the assembly (Mode 2 assembly only).")
 
@@ -909,9 +955,6 @@ void PalindromicReadOptions::write(ostream& s) const
     s << "palindromicReads.alignedFractionThreshold = " << alignedFractionThreshold << "\n";
     s << "palindromicReads.nearDiagonalFractionThreshold = " << nearDiagonalFractionThreshold << "\n";
     s << "palindromicReads.deltaThreshold = " << deltaThreshold << "\n";
-    s << "palindromicReads.qScoreRelativeMeanDifference = " << qScoreRelativeMeanDifference << "\n";
-    s << "palindromicReads.qScoreMinimumMean = " << qScoreMinimumMean << "\n";
-    s << "palindromicReads.qScoreMinimumVariance = " << qScoreMinimumVariance << "\n";
 }
 
 
@@ -919,6 +962,7 @@ void PalindromicReadOptions::write(ostream& s) const
 void ReadsOptions::write(ostream& s) const
 {
     s << "[Reads]\n";
+    s << "representation = " << representation << "\n";
     s << "minReadLength = " << minReadLength << "\n";
     s << "desiredCoverage = " << desiredCoverageString << "\n";
     s << "noCache = " <<
@@ -1033,11 +1077,14 @@ void MarkerGraphOptions::write(ostream& s) const
     s << "pruneIterationCount = " << pruneIterationCount << "\n";
     s << "simplifyMaxLength = " << simplifyMaxLength << "\n";
     s << "crossEdgeCoverageThreshold = " << crossEdgeCoverageThreshold << "\n";
-    s << "refineThreshold = " << refineThreshold << "\n";
     s << "reverseTransitiveReduction = " <<
         convertBoolToPythonString(reverseTransitiveReduction) << "\n";
     s << "peakFinder.minAreaFraction = " << peakFinderMinAreaFraction << "\n";
     s << "peakFinder.areaStartIndex = " << peakFinderAreaStartIndex << "\n";
+
+    s << "secondaryEdges.maxSkip = " << secondaryEdgesMaxSkip << "\n";
+    s << "secondaryEdges.split.errorRateThreshold = " << secondaryEdgesSplitErrorRateThreshold << "\n";
+    s << "secondaryEdges.split.minCoverage = " << secondaryEdgesSplitMinCoverage << "\n";
 }
 
 
@@ -1057,6 +1104,7 @@ void AssemblyOptions::write(ostream& s) const
         storeCoverageDataCsvLengthThreshold << "\n";
     s << "writeReadsByAssembledSegment = " <<
         convertBoolToPythonString(writeReadsByAssembledSegment) << "\n";
+    s << "pruneLength = " << pruneLength << "\n";
     s << "detangleMethod = " << detangleMethod << "\n";
     s << "detangle.diagonalReadCountMin = " << detangleDiagonalReadCountMin << "\n";
     s << "detangle.offDiagonalReadCountMax = " << detangleOffDiagonalReadCountMax << "\n";
@@ -1073,17 +1121,32 @@ void AssemblyOptions::write(ostream& s) const
     s << "iterative.bridgeRemovalIterationCount = " << iterativeBridgeRemovalIterationCount << "\n";
     s << "iterative.bridgeRemovalMaxDistance = " << iterativeBridgeRemovalMaxDistance << "\n";
 
-    s << "bubbleRemoval.discordantRatioThreshold = " << bubbleRemovalDiscordantRatioThreshold << "\n";
-    s << "bubbleRemoval.ambiguityThreshold = " << bubbleRemovalAmbiguityThreshold << "\n";
-    s << "bubbleRemoval.maxPeriod = " << bubbleRemovalMaxPeriod << "\n";
-    s << "superbubbleRemoval.edgeLengthThreshold = " << superbubbleRemovalEdgeLengthThreshold << "\n";
-    s << "phasing.minReadCount = " << phasingMinReadCount << "\n";
+    mode2Options.write(s);
+}
 
-    s << "suppressGfaOutput = " << convertBoolToPythonString(suppressGfaOutput) << "\n";
-    s << "suppressFastaOutput = " << convertBoolToPythonString(suppressFastaOutput) << "\n";
-    s << "suppressDetailedOutput = " << convertBoolToPythonString(suppressDetailedOutput) << "\n";
-    s << "suppressPhasedOutput = " << convertBoolToPythonString(suppressPhasedOutput) << "\n";
-    s << "suppressHaploidOutput = " << convertBoolToPythonString(suppressHaploidOutput) << "\n";
+
+
+void Mode2AssemblyOptions::write(ostream& s) const
+{
+    s << "mode2.strongBranchThreshold = " << strongBranchThreshold << "\n";
+    s << "mode2.epsilon = " << epsilon << "\n";
+    s << "mode2.bubbleRemoval.minConcordantReadCount = " << minConcordantReadCountForBubbleRemoval << "\n";
+    s << "mode2.bubbleRemoval.maxDiscordantReadCount = " << maxDiscordantReadCountForBubbleRemoval << "\n";
+    s << "mode2.bubbleRemoval.minLogP = " << minLogPForBubbleRemoval << "\n";
+    s << "mode2.bubbleRemoval.componentSizeThreshold = " << componentSizeThresholdForBubbleRemoval << "\n";
+    s << "mode2.phasing.minConcordantReadCount = " << minConcordantReadCountForPhasing << "\n";
+    s << "mode2.phasing.maxDiscordantReadCount = " << maxDiscordantReadCountForPhasing << "\n";
+    s << "mode2.phasing.minLogP = " << minLogPForPhasing << "\n";
+    s << "mode2.superbubble.maxSize = " << maxSuperbubbleSize << "\n";
+    s << "mode2.superbubble.maxChunkSize = " << maxSuperbubbleChunkSize << "\n";
+    s << "mode2.superbubble.maxChunkPathCount = " << maxSuperbubbleChunkPathCount << "\n";
+    s << "mode2.superbubble.edgeLengthThreshold = " << superbubbleEdgeLengthThreshold << "\n";
+
+    s << "mode2.suppressGfaOutput = " << convertBoolToPythonString(suppressGfaOutput) << "\n";
+    s << "mode2.suppressFastaOutput = " << convertBoolToPythonString(suppressFastaOutput) << "\n";
+    s << "mode2.suppressDetailedOutput = " << convertBoolToPythonString(suppressDetailedOutput) << "\n";
+    s << "mode2.suppressPhasedOutput = " << convertBoolToPythonString(suppressPhasedOutput) << "\n";
+    s << "mode2.suppressHaploidOutput = " << convertBoolToPythonString(suppressHaploidOutput) << "\n";
 }
 
 
@@ -1114,7 +1177,7 @@ void MarkerGraphOptions::parseSimplifyMaxLength()
 
     boost::tokenizer< boost::char_separator<char> > tokenizer(
         simplifyMaxLength, boost::char_separator<char>(","));
-    for(const string token: tokenizer) {
+    for(const string& token: tokenizer) {
         try {
             size_t numberEndsHere;
             const size_t value = std::stoi(token, &numberEndsHere);
