@@ -66,19 +66,19 @@ bool Assembler::passesReadGraph2Criteria(const AlignmentInfo& info) const{
     const auto trim = max(trims.first, trims.second);
 
     // If this alignment doesn't pass the thresholds, skip it
-    if (info.minAlignedFraction() < assemblerInfo->automatedAlignedFractionThreshold){
+    if (info.minAlignedFraction() < assemblerInfo->actualMinAlignedFraction){
         return false;
     }
-    if (info.markerCount < assemblerInfo->automatedMarkerCountThreshold){
+    if (info.markerCount < assemblerInfo->actualMinAlignedMarkerCount){
         return false;
     }
-    if (info.maxDrift > assemblerInfo->automatedMaxDriftThreshold){
+    if (info.maxDrift > assemblerInfo->actualMaxDrift){
         return false;
     }
-    if (info.maxSkip > assemblerInfo->automatedMaxSkipThreshold){
+    if (info.maxSkip > assemblerInfo->actualMaxSkip){
         return false;
     }
-    if (trim > assemblerInfo->automatedMaxTrimThreshold){
+    if (trim > assemblerInfo->actualMaxTrim){
         return false;
     }
 
@@ -138,26 +138,26 @@ void Assembler::setReadGraph2Criteria(
         }
     }
 
-    // Minimums
-    assemblerInfo->automatedAlignedFractionThreshold =
+    // Minimums.
+    assemblerInfo->actualMinAlignedFraction =
             alignedFractionHistogram.thresholdByCumulativeProportion(alignedFractionPercentile);
-    assemblerInfo->automatedMarkerCountThreshold =
-            markerCountHistogram.thresholdByCumulativeProportion(markerCountPercentile);
+    assemblerInfo->actualMinAlignedMarkerCount =
+            uint64_t(std::round(markerCountHistogram.thresholdByCumulativeProportion(markerCountPercentile)));
 
     // Maximums use (1 - percentile)
-    assemblerInfo->automatedMaxDriftThreshold =
-            maxDriftHistogram.thresholdByCumulativeProportion(1 - maxDriftPercentile);
-    assemblerInfo->automatedMaxSkipThreshold =
-            maxSkipHistogram.thresholdByCumulativeProportion(1 - maxSkipPercentile);
-    assemblerInfo->automatedMaxTrimThreshold =
-            maxTrimHistogram.thresholdByCumulativeProportion(1 - maxTrimPercentile);
+    assemblerInfo->actualMaxDrift =
+        uint64_t(std::round(maxDriftHistogram.thresholdByCumulativeProportion(1 - maxDriftPercentile)));
+    assemblerInfo->actualMaxSkip =
+        uint64_t(std::round(maxSkipHistogram.thresholdByCumulativeProportion(1 - maxSkipPercentile)));
+    assemblerInfo->actualMaxTrim =
+        uint64_t(std::round(maxTrimHistogram.thresholdByCumulativeProportion(1 - maxTrimPercentile)));
 
-    cout << "Selected thresholds automatically for the following parameters:\n\t"
-         << "alignedFraction:\t" << assemblerInfo->automatedAlignedFractionThreshold << "\n\t"
-         << "markerCount:\t\t" << assemblerInfo->automatedMarkerCountThreshold << "\n\t"
-         << "maxDrift:\t\t" << assemblerInfo->automatedMaxDriftThreshold << "\n\t"
-         << "maxSkip:\t\t" << assemblerInfo->automatedMaxSkipThreshold << "\n\t"
-         << "maxTrim:\t\t" << assemblerInfo->automatedMaxTrimThreshold << "\n";
+    cout << "Automatically selected alignment criteria:\n\t"
+         << "minAlignedFraction:\t" << assemblerInfo->actualMinAlignedFraction << "\n\t"
+         << "minAlignedMarkerCount:\t\t" << assemblerInfo->actualMinAlignedMarkerCount << "\n\t"
+         << "maxDrift:\t\t" << assemblerInfo->actualMaxDrift << "\n\t"
+         << "maxSkip:\t\t" << assemblerInfo->actualMaxSkip << "\n\t"
+         << "maxTrim:\t\t" << assemblerInfo->actualMaxTrim << "\n";
 
     ofstream alignedFractionHistogramCsv("AlignedFractionHistogram.csv");
     ofstream markerCountHistogramCsv("AlignmentMarkerCountHistogram.csv");
