@@ -1,9 +1,7 @@
 #ifndef SHASTA_ASSEMBLY_GRAPH2_HPP
 #define SHASTA_ASSEMBLY_GRAPH2_HPP
 
-// Assembly graph for assembly mode2.
-
-
+// Assembly graph for assembly mode 2.
 
 // Shasta.
 #include "Marker.hpp"
@@ -14,7 +12,6 @@
 #include <boost/graph/adjacency_list.hpp>
 
 // Standard library.
-#include "array.hpp"
 #include <map>
 #include "string.hpp"
 #include "vector.hpp"
@@ -25,8 +22,10 @@ namespace shasta {
     class AssemblyGraph2;
     class AssemblyGraph2Vertex;
     class AssemblyGraph2Edge;
+    class AssemblyGraph2Statistics;
     class BubbleChain;
     class MarkerGraph;
+    class Mode2AssemblyOptions;
     class ReadFlags;
 
     using AssemblyGraph2BaseClass =
@@ -35,6 +34,8 @@ namespace shasta {
 
     using MarkerGraphPath = vector<MarkerGraph::EdgeId>;
 
+    class AssemblerInfo;
+
     namespace MemoryMapped {
         template<class T, class Int> class VectorOfVectors;
     }
@@ -94,6 +95,7 @@ public:
         Branch(const MarkerGraphPath& path, bool containsSecondaryEdges) :
             path(path),
             containsSecondaryEdges(containsSecondaryEdges) {}
+        Branch() {}
 
         // Assembled sequence.
         // This excludes the first and last k/2 RLE bases.
@@ -108,8 +110,8 @@ public:
 
         // Minimum and sum of coverage on the marker graph graph
         // edges of this branch.
-        uint64_t minimumCoverage;
-        uint64_t coverageSum;
+        uint64_t minimumCoverage = 0;
+        uint64_t coverageSum = 0;
         uint64_t averageCoverage() const
         {
             return uint64_t(std::round(double(coverageSum) / double(path.size())));
@@ -121,12 +123,12 @@ public:
     vector<Branch> branches;
 
     // The strongest branch.
-    uint64_t strongestBranchId;
-    void findStrongestBranch();
+    uint64_t getStrongestBranchId() const;
 
     // Functions that remove some of the branches.
     void removeAllSecondaryBranches();
     void removeAllBranchesExceptStrongest();
+    void forceMaximumPloidy(uint64_t);
 
     // Store read information on all branches.
     void storeReadInformation(const MarkerGraph&);
@@ -186,11 +188,24 @@ public:
         return length;
     }
 
+    uint64_t minimumPathLength() const
+    {
+        uint64_t length = std::numeric_limits<uint64_t>::max();
+        for(const Branch& branch: branches) {
+            length = min(length, uint64_t(branch.path.size()));
+        }
+        return length;
+    }
+
     // Return the number of raw bases of sequence identical between
     // all branches at the beginning/end.
     uint64_t countCommonPrefixBases() const;
     uint64_t countCommonSuffixBases() const;
 
+    // Compute the edit distance between the sequences of the two branches.
+    // This can only be called for a diploid bubble (2 branches).
+    uint64_t bubbleEditDistance() const;
+
     // The number of raw sequence bases transfered
     // in each direction for gfa output.
     uint64_t backwardTransferCount = 0;
@@ -210,8 +225,14 @@ public:
 
     // Phasing information - only set for bubbles not marker
     // as bad by the above flag.
-    uint64_t componentId = std::numeric_limits<uint64_t>::max();
-    uint64_t phase = std::numeric_limits<uint64_t>::max();
+    static const uint64_t invalidComponentId = std::numeric_limits<uint64_t>::max();
+    uint64_t componentId = invalidComponentId;
+    static const uint64_t invalidPhase = std::numeric_limits<uint64_t>::max();
+    uint64_t phase = invalidPhase;
+    bool isPhased() const
+    {
+        return componentId != std::numeric_limits<uint64_t>::max();
+    }
 
     // Bubble chain information.
     // If this edge is part of a bubble chain, this stores
@@ -231,21 +252,16 @@ public:
     // in the marker graph. Therefore, immediately after construction,
     // each edge has a single MarkerGraphPath (no bubbles).
     AssemblyGraph2(
+        uint64_t readRepresentation,
         uint64_t k, // Marker length
         const MemoryMapped::Vector<ReadFlags>& readFlags,
         const MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t>& markers,
-        const MarkerGraph&,
-        double bubbleRemovalDiscordantRatioThreshold,
-        double bubbleRemovalAmbiguityThreshold,
-        uint64_t bubbleRemovalMaxPeriod,
-        uint64_t superbubbleRemovalEdgeLengthThreshold,
-        uint64_t phasingMinReadCount,
-        bool suppressGfaOutput,
-        bool suppressFastaOutput,
-        bool suppressSmallBubblesOutput,
-        bool suppressPhasedOutput,
-        bool suppressHaploidOutput,
-        size_t threadCount
+        MarkerGraph&,
+        uint64_t pruneLength,
+        const Mode2AssemblyOptions&,
+        AssemblyGraph2Statistics&,
+        size_t threadCount,
+        bool debug
         );
 
     void writeCsv(const string& baseName) const;
@@ -270,13 +286,16 @@ public:
         bool writeSequence,
         bool writeCsv,
         bool writeGfa,
-        bool writeFasta) const;
+        bool writeFasta,
+        AssemblyGraph2Statistics* statistics = 0) const;
     void writePhased(
         const string& baseName,
         bool writeSequence,
         bool writeCsv,
         bool writeGfa,
-        bool writeFasta) const;
+        bool writeFasta,
+        AssemblyGraph2Statistics* statistics = 0) const;
+    void writePhasedDetails() const;
 
     // Hide a AssemblyGraph2BaseClass::Base.
     using Base = shasta::Base;
@@ -289,10 +308,17 @@ private:
     using E = AssemblyGraph2Edge;
 
     // Some Assembler data that we need.
+    uint64_t readRepresentation;
     uint64_t k;
     const MemoryMapped::Vector<ReadFlags>& readFlags;
     const MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t>& markers;
-    const MarkerGraph& markerGraph;
+public:
+    uint64_t getReadCount() const
+    {
+        return markers.size() / 2;
+    }
+private:
+    MarkerGraph& markerGraph;
 
     // Map that gives us the vertex descriptor corresponding to
     // each marker graph vertex.
@@ -301,14 +327,50 @@ private:
     // Initial creation of vertices and edges.
     void create();
 
-    // Remove secondary edges making sure to not introduce any dead ends.
-    void cleanupSecondaryEdges();
+    // Prune leafs in which all branches are shorter than the specified length.
+    void prune(uint64_t pruneLength);
+
+
+
+    // Superbubble removal.
+
+    // This creates superbubbles using edges shorter than a length threshold (in markers).
+    void handleSuperbubbles0(
+        uint64_t edgeLengthThreshold,
+        uint64_t maxSuperbubbleSize,
+        uint64_t maxSuperbubbleChunkSize,
+        uint64_t maxSuperbubbleChunkPathCount,
+        bool storeReadInformation,  // If true, store read information for newly created edges.
+        bool assemble               // If true, assemble sequence for newly created edges
+        );
+
+    // This creates superbubbles using all edges not in bubble chains.
+    void handleSuperbubbles1(
+        uint64_t maxSuperbubbleSize,
+        uint64_t maxSuperbubbleChunkSize,
+        uint64_t maxSuperbubbleChunkPathCount,
+        bool storeReadInformation,  // If true, store read information for newly created edges.
+        bool assemble               // If true, assemble sequence for newly created edges
+        );
 
-    // Handle superbubbles.
-    void handleSuperbubbles(uint64_t edgeLengthThreshold);
     class Superbubble;
-    void handleSuperbubble0(Superbubble&);
-    void handleSuperbubble1(Superbubble&);
+
+    // This uses a dominator tree to find choking points
+    // and partition the superbubble into chunks,
+    // then does path enumeration on individual chunks.
+    void handleSuperbubble1(
+        Superbubble&,
+        uint64_t maxSuperbubbleSize,
+        uint64_t maxSuperbubbleChunkSize,
+        uint64_t maxSuperbubbleChunkPathCount,
+        bool storeReadInformation,  // If true, store read information for newly created edges.
+        bool assemble               // If true, assemble sequence for newly created edges
+        );
+
+
+
+    // Remove short loop-back edges.
+    void removeShortLoopbackEdges(uint64_t edgeLengthThreshold);
 
     // Remove degenerate branches
     void removeDegenerateBranches();
@@ -324,7 +386,10 @@ private:
     // Also create the vertices if necessary.
     edge_descriptor addEdge(
         const MarkerGraphPath&,
-        bool containsSecondaryEdges);
+        bool containsSecondaryEdges,
+        bool storeReadInformation,  // If true, store read information for newly created edges.
+        bool assemble               // If true, assemble sequence for newly created edges
+        );
 
     // Merge consecutive non-bubbles, when possible.
      void merge(
@@ -337,10 +402,19 @@ private:
         bool assemble               // If true, assemble merged edges.
         );
 
+    // Merge an edge with the previous or following edge, if possible.
+    edge_descriptor mergeWithPreviousIfPossible(edge_descriptor);
+    edge_descriptor mergeWithFollowingIfPossible(edge_descriptor);
+
     // Find linear chains of adjacent non-bubbles.
     // Used by merge.
     void findNonBubbleLinearChains(vector< vector<edge_descriptor> >&) const;
 
+    // Find and create new bubbles.
+    void createNewBubbles();
+
+    void writePloidyHistogram(ostream&) const;
+
 
 
     // Predicate used to select non-bubble edges.
@@ -361,6 +435,13 @@ private:
 
     // Assemble sequence for every marker graph path of every edge.
     void assemble();
+    void assembleParallel(uint64_t threadCount);
+    void assembleThreadFunction(size_t threadId);
+    class AssembleParallelData {
+    public:
+        vector<edge_descriptor> allEdges;
+    };
+    AssembleParallelData assembleParallelData;
 
     // Assemble sequence for every marker graph path of a given edge.
     void assemble(edge_descriptor);
@@ -370,7 +451,8 @@ private:
 
     void hetSnpStatistics(
         uint64_t& transitionCount,
-        uint64_t& transversionCount
+        uint64_t& transversionCount,
+        uint64_t& nonSnpCount
     ) const;
 
     // Finds edges that form bubbles, then combine
@@ -381,16 +463,10 @@ private:
         vertex_descriptor v1,
         const vector<edge_descriptor>&);
 
-    void removeSecondaryBubbles();
+    void removeSecondaryBubbles(uint64_t secondaryEdgeCleanupThreshold);
+    void removeWeakBranches(uint64_t strongBranchThreshold);
 
-    // Find/remove bubbles caused by copy number changes in repeats
-    // with period up to maxPeriod.
-    void findCopyNumberBubbles(uint64_t maxPeriod);
-    void removeCopyNumberBubbles();
-
-    // Remove bubbles marked isBad during phasing.
-    // Only keep the strongest branch for each.
-    void removeBadBubbles();
+    void forceMaximumPloidy(uint64_t maxPloidy);
 
     // For each edge, compute the number of raw sequence bases
     // transfered in each direction for gfa output.
@@ -400,16 +476,28 @@ private:
 
     // Store read information on all edges.
     void storeReadInformation();
+    void storeReadInformationParallel(uint64_t threadCount);
+    void storeReadInformationThreadFunction(size_t threadId);
+    class StoreReadInformationParallelData {
+    public:
+        vector<edge_descriptor> allEdges;
+    };
+    StoreReadInformationParallelData storeReadInformationParallelData;
 
 
     // Linear chains of bubbles in the AssemblyGraph2.
     vector<BubbleChain> bubbleChains;
     void findBubbleChains();
+    void clearBubbleChains();
     void writeBubbleChains();
     void findPhasingRegions();
     void findPhasingRegions(BubbleChain&);
     void writePhasingRegions();
 
+    // Set the wasAssembled flag for marker graph edges that are
+    // in the final AssemblyGraph2.
+    void updateMarkerGraph();
+
     // Compute the gfa sequence of a bubble chain
     // by concatenating gfa sequence of the strongest branch of
     // each of this edges.
@@ -490,6 +578,10 @@ private:
             const vector<AssemblyGraph2::vertex_descriptor>&,
             uint64_t edgeLengthThreshold
         );
+        Superbubble(
+            const AssemblyGraph2&,
+            const vector<AssemblyGraph2::vertex_descriptor>&
+        );
         vector<Superbubble::vertex_descriptor> entrances;
         vector<Superbubble::vertex_descriptor> exits;
         bool isSimpleLinearChain() const;
@@ -525,241 +617,30 @@ private:
 
 
 
-    // Graph used for diploid phasing of the bubbles.
-    // Each vertex corresponds to a diploid bubble in the Assembly2Graph.
+    // Iteratively remove bad bubbles using the PhasingGraph.
+    void removeBadBubblesIterative(
+        uint64_t minConcordantReadCount,
+        uint64_t maxDiscordantReadCount,
+        double minLogP,
+        double epsilon,
+        uint64_t superbubbleRemovalEdgeLengthThreshold,
+        uint64_t maxSuperbubbleSize,
+        uint64_t maxSuperbubbleChunkSize,
+        uint64_t maxSuperbubbleChunkPathCount,
+        uint64_t pruneLength,
+        uint64_t componentSizeThreshold,
+        size_t threadCount);
 
-    class BubbleGraphVertex {
-    public:
-        AssemblyGraph2::edge_descriptor e;
-        uint64_t id;    // The same as the id of the AssemblyGraph2Edge.
-        BubbleGraphVertex(
-            AssemblyGraph2::edge_descriptor,
-            const AssemblyGraph2Edge&);
+    // Hierarchical phasing using the PhasingGraph.
+    void hierarchicalPhase(
+        uint64_t minConcordantReadCount,
+        uint64_t maxDiscordantReadCount,
+        double minLogP,
+        double epsilon,
+        size_t threadCount);
 
-        // The vertex stores OrientedReadIds that appear on one side but not
-        // on the opposite side of the bubble.
-        // Stored as pairs (OrientedReadId, side).
-        vector< pair<OrientedReadId, uint64_t> > orientedReadIds;
-
-        // The connected component this vertex belongs to.
-        uint64_t componentId;
-
-        // The phase assigned to this vertex (bubble)
-        // It is only meaningful within each connected component.
-        static const uint64_t invalidPhase = std::numeric_limits<uint64_t>::max();
-        uint64_t phase = invalidPhase;
-
-        // Layout position for display.
-        array<double, 2> position;
-
-
-
-    };
-
-
-
-    class BubbleGraphEdge {
-    public:
-        // Store the number of common oriented reads for each pair of
-        // branches in the two bubbles.
-        // matrix[sideA][sideB] stores the number of OrientedReadIds
-        // that appear on sideA of the "first" bubble and on
-        // sideB of the "second" bubble of this edge.
-        // The "first" bubble of the edge is the lowered numbered.
-        array<array<uint64_t, 2>, 2> matrix;
-        BubbleGraphEdge()
-        {
-            for(uint64_t i=0; i<2; i++) {
-                for(uint64_t j=0; j<2; j++) {
-                    matrix[i][j] = 0;
-                }
-            }
-        }
-
-        uint64_t diagonalCount() const
-        {
-            return matrix[0][0] + matrix[1][1];
-        }
-        uint64_t offDiagonalCount() const
-        {
-            return matrix[0][1] + matrix[1][0];
-        }
-        uint64_t totalCount() const
-        {
-            return diagonalCount() + offDiagonalCount();
-        }
-        uint64_t concordantCount() const
-        {
-            return max(diagonalCount(), offDiagonalCount());
-        }
-        uint64_t discordantCount() const
-        {
-            return min(diagonalCount(), offDiagonalCount());
-        }
-
-        // Ambiguity of the edge is 0 if discordantCount() is 0
-        // and 1 if discordantCount() = totalCount()/2, in which case
-        // discordantCount() = concordantCount();
-        double ambiguity() const
-        {
-            return double(2 * discordantCount()) / double(totalCount());
-        }
-
-        // Return the relative phase implied by this edge, which is
-        // +1 if offdiagonalCount() is 0 and
-        // -1 if diagonalCount() is 0.
-        double relativePhase() const
-        {
-            const double diagonalRatio = double(diagonalCount()) / double(totalCount());
-            return 2. * diagonalRatio - 1.;
-        }
-    };
-
-
-
-    // Bubble graph.
-    // It is an undirected graph where each vertex represents a bubble.
-    using BubbleGraphBaseClass =
-        boost::adjacency_list<boost::listS, boost::listS, boost::undirectedS,
-        BubbleGraphVertex, BubbleGraphEdge>;
-
-    class BubbleGraph:
-        public BubbleGraphBaseClass,
-        public MultithreadedObject<BubbleGraph> {
-    public:
-        BubbleGraph() : MultithreadedObject<BubbleGraph>(*this) {}
-
-        // A table that, for each OrientedReadId, contains a list of
-        // pairs(vertex, side) that the OrientedReadId appears on.
-        // Indexed by OrientedReadId::getValue().
-        vector< vector< pair<BubbleGraph::vertex_descriptor, uint64_t> > > orientedReadsTable;
-        void createOrientedReadsTable(uint64_t readCount);
-        void writeEdgesCsv(const string& fileName) const;
-        void removeWeakEdges(uint64_t minReadCount);
-        double discordantRatio(vertex_descriptor) const;
-        void removeWeakVertices(
-            double discordantRatioThreshold,
-            vector<AssemblyGraph2::edge_descriptor>& badBubbles);
-
-        // Pre-phasing.
-        void writeHtml(const string& fileName);
-        void writeGraphviz(const string& fileName) const;
-
-        // Post-phasing.
-        void writeHtml(
-            const string& fileName,
-            const std::set<edge_descriptor>& treeEdges,
-            bool onlyShowUnhappyEdges);
-
-        void computeConnectedComponents();
-        vector< vector<BubbleGraph::vertex_descriptor> > connectedComponents;
-
-        // Create a new BubbleGraph from a given connected component.
-        void extractComponent(uint64_t componentId, BubbleGraph&) const;
-
-        // Return true if the give edge has relative phase consistent
-        // with the phases assigned to its two vertices.
-        bool edgeIsHappy(edge_descriptor e) const;
-
-
-        // Edge creation is expensive.
-        // There is a simple sequential version and a more complex
-        // parallel version.
-        void createEdges(uint64_t phasingMinReadCount);
-        void createEdgesParallel(
-            uint64_t phasingMinReadCount,
-            size_t threadCount);
-        void createEdgesParallelThreadFunction(size_t threadId);
-        class CreateEdgesParallelData {
-        public:
-            uint64_t phasingMinReadCount;
-            vector<BubbleGraph::vertex_descriptor> allVertices;
-            class EdgeData {
-            public:
-                BubbleGraph::vertex_descriptor vB;
-                uint64_t sideA;
-                uint64_t sideB;
-                bool operator<(const EdgeData& that) const
-                {
-                    return vB < that.vB;
-                }
-            };
-        };
-        CreateEdgesParallelData createEdgesParallelData;
-        void createEdges(
-            BubbleGraph::vertex_descriptor,
-            uint64_t phasingMinReadCount,
-            vector<CreateEdgesParallelData::EdgeData>&);
-    };
-
-
-
-    BubbleGraph bubbleGraph;
-    void createBubbleGraph(
-        uint64_t readCount,             // Total.
-        uint64_t phasingMinReadCount,   // For an edge to be kept.
-        size_t threadCount
-        );
-    void cleanupBubbleGraph(
-        double discordantRatioThreshold,
-        double ambiguityThreshold);
-
-
-
-    // A predicate used to filter BubbleGraph edges for which
-    // relativePhase() >= minRelativePhase.
-    // This is only used for html output of the BubbleGraph.
-    class BubbleGraphEdgePredicate1 {
-    public:
-        BubbleGraphEdgePredicate1(
-            const BubbleGraph& bubbleGraph,
-            double minRelativePhase) :
-            bubbleGraph(&bubbleGraph),
-            minRelativePhase(minRelativePhase) {}
-
-        const BubbleGraph* bubbleGraph;
-        double minRelativePhase;
-
-        bool operator() (const BubbleGraph::edge_descriptor e) const
-        {
-            return (*bubbleGraph)[e].relativePhase() >= minRelativePhase;
-        }
-    };
-
-
-
-    // A predicate used to filter BubbleGraph spanning tree edges
-    // and edges for which relativePhase() >= minRelativePhase.
-    // This is only used for graphviz output of each connectec component, post-phasing.
-    class BubbleGraphEdgePredicate2 {
-    public:
-        BubbleGraphEdgePredicate2(
-            const BubbleGraph& bubbleGraph,
-            double minRelativePhase,
-            const std::set<BubbleGraph::edge_descriptor>& treeEdges) :
-            bubbleGraph(&bubbleGraph),
-            minRelativePhase(minRelativePhase),
-            treeEdges(&treeEdges) {}
-
-        const BubbleGraph* bubbleGraph;
-        double minRelativePhase;
-        const std::set<BubbleGraph::edge_descriptor>* treeEdges;
-
-        bool operator() (const BubbleGraph::edge_descriptor e) const
-        {
-            return
-                (*bubbleGraph)[e].relativePhase() >= minRelativePhase
-                or
-                treeEdges->find(e) != treeEdges->end();
-        }
-    };
-
-
-
-    // Use each connected component of the bubble graph to phase the bubbles.
-    void phase(size_t threadCount);
-    void phaseThreadFunction(size_t threadId);
-    void phaseBubbleGraphComponent(uint64_t componentId);
+    // Renumber component to make them contiguous starting at 0.
+    void renumberComponents();
 
 };
 
