@@ -10,18 +10,18 @@
 #include "compressAlignment.hpp"
 #include "deduplicate.hpp"
 #include "dset64Test.hpp"
+#include "diploidBayesianPhase.hpp"
 #include "shastaLapack.hpp"
 #include "LongBaseSequence.hpp"
 #include "mappedCopy.hpp"
-#include "Map.hpp"
 #include "MedianConsensusCaller.hpp"
 #include "MemoryMappedAllocator.hpp"
 #include "MultithreadedObject.hpp"
+#include "performanceLog.hpp"
 #include "Reads.hpp"
 #include "ShortBaseSequence.hpp"
 #include "splitRange.hpp"
 #include "testSpoa.hpp"
-#include "shortestPathBoundedDistance.hpp"
 #include "SimpleBayesianConsensusCaller.hpp"
 using namespace shasta;
 
@@ -33,17 +33,17 @@ using namespace pybind11;
 
 
 
-PYBIND11_MODULE(shasta, module)
+PYBIND11_MODULE(shasta, shastaModule)
 {
 
     // Expose class OrientedReadPair to Python.
-    class_<OrientedReadPair>(module, "OrientedReadPair")
+    class_<OrientedReadPair>(shastaModule, "OrientedReadPair")
         .def_readonly("readIds", &OrientedReadPair::readIds)
         .def_readonly("isSameStrand", &OrientedReadPair::isSameStrand)
         ;
 
     // Expose class Reads to Python
-    class_<Reads>(module, "Reads")
+    class_<Reads>(shastaModule, "Reads")
         .def("readCount", &Reads::readCount, "Get the number of reads.")
         .def("writeReads",
             &Reads::writeReads,
@@ -80,7 +80,7 @@ PYBIND11_MODULE(shasta, module)
 
 
     // Expose class AlignOptions to Python.
-    class_<AlignOptions>(module, "AlignOptions")
+    class_<AlignOptions>(shastaModule, "AlignOptions")
         .def(pybind11::init<>())
         .def_readwrite("alignMethod", &AlignOptions::alignMethod)
         .def_readwrite("maxSkip", &AlignOptions::maxSkip)
@@ -104,16 +104,40 @@ PYBIND11_MODULE(shasta, module)
         .def_readwrite("align4MaxDistanceFromBoundary", &AlignOptions::align4MaxDistanceFromBoundary)
         ;
 
+    // Expose class Mode2AssemblyOptions to Python.
+    class_<Mode2AssemblyOptions>(shastaModule, "Mode2AssemblyOptions")
+        .def(pybind11::init<>())
+        .def_readwrite("strongBranchThreshold", &Mode2AssemblyOptions::strongBranchThreshold)
+        .def_readwrite("epsilon", &Mode2AssemblyOptions::epsilon)
+        .def_readwrite("minConcordantReadCountForBubbleRemoval", &Mode2AssemblyOptions::minConcordantReadCountForBubbleRemoval)
+        .def_readwrite("maxDiscordantReadCountForBubbleRemoval", &Mode2AssemblyOptions::maxDiscordantReadCountForBubbleRemoval)
+        .def_readwrite("minLogPForBubbleRemoval", &Mode2AssemblyOptions::minLogPForBubbleRemoval)
+        .def_readwrite("componentSizeThresholdForBubbleRemoval", &Mode2AssemblyOptions::componentSizeThresholdForBubbleRemoval)
+        .def_readwrite("minConcordantReadCountForPhasing", &Mode2AssemblyOptions::minConcordantReadCountForPhasing)
+        .def_readwrite("maxDiscordantReadCountForPhasing", &Mode2AssemblyOptions::maxDiscordantReadCountForPhasing)
+        .def_readwrite("minLogPForPhasing", &Mode2AssemblyOptions::minLogPForPhasing)
+        .def_readwrite("maxSuperbubbleSize", &Mode2AssemblyOptions::maxSuperbubbleSize)
+        .def_readwrite("maxSuperbubbleChunkSize", &Mode2AssemblyOptions::maxSuperbubbleChunkSize)
+        .def_readwrite("maxSuperbubbleChunkPathCount", &Mode2AssemblyOptions::maxSuperbubbleChunkPathCount)
+        .def_readwrite("superbubbleEdgeLengthThreshold", &Mode2AssemblyOptions::superbubbleEdgeLengthThreshold)
+        .def_readwrite("suppressGfaOutput", &Mode2AssemblyOptions::suppressGfaOutput)
+        .def_readwrite("suppressFastaOutput", &Mode2AssemblyOptions::suppressFastaOutput)
+        .def_readwrite("suppressDetailedOutput", &Mode2AssemblyOptions::suppressDetailedOutput)
+        .def_readwrite("suppressPhasedOutput", &Mode2AssemblyOptions::suppressPhasedOutput)
+        .def_readwrite("suppressHaploidOutput", &Mode2AssemblyOptions::suppressHaploidOutput)
+        ;
+
 
 
     // Expose class Assembler to Python.
-    class_<Assembler>(module, "Assembler")
+    class_<Assembler>(shastaModule, "Assembler")
 
         // Constructor.
-        .def(pybind11::init<const string&, bool, size_t>(),
+        .def(pybind11::init<const string&, bool, uint64_t, size_t>(),
             "Assembler constructor.",
             arg("largeDataFileNamePrefix") = "Data/",
             arg("createNew") = false,
+            arg("readRepresentation") = 1,
             arg("largeDataPageSize") = 2*1024*1024)
 
 
@@ -230,7 +254,6 @@ PYBIND11_MODULE(shasta, module)
             arg("alignedFractionThreshold"),
             arg("nearDiagonalFractionThreshold"),
             arg("deltaThreshold"),
-            arg("writeTocsv") = false,
             arg("threadCount") = 0)
 
         // Alignments.
@@ -356,10 +379,6 @@ PYBIND11_MODULE(shasta, module)
              arg("maxAlignmentCount"),
              arg("threadCount") = 1
              )
-         .def("createReadGraphMode1",
-              &Assembler::createReadGraphMode1,
-              arg("maxAlignmentCount")
-              )
         .def("accessReadGraph",
             &Assembler::accessReadGraph)
         .def("accessReadGraphReadWrite",
@@ -448,10 +467,6 @@ PYBIND11_MODULE(shasta, module)
             arg("threadCount") = 0)
         .def("computeMarkerGraphCoverageHistogram",
             &Assembler::computeMarkerGraphCoverageHistogram)
-        .def("refineMarkerGraph",
-            &Assembler::refineMarkerGraph,
-            arg("refineThreshold"),
-            arg("threadCount") = 0)
         .def("writeBadMarkerGraphVertices",
             &Assembler::writeBadMarkerGraphVertices)
         .def("cleanupDuplicateMarkers",
@@ -466,8 +481,6 @@ PYBIND11_MODULE(shasta, module)
             &Assembler::getMarkerGraphMinCoverageUsed)
         .def("vertexCoverageStatisticsByKmerId",
             &Assembler::vertexCoverageStatisticsByKmerId)
-        .def("analyzeMarkerGraphForks",
-            &Assembler::analyzeMarkerGraphForks)
 
         // Edges of the global marker graph.
         .def("createMarkerGraphEdges",
@@ -485,11 +498,21 @@ PYBIND11_MODULE(shasta, module)
             &Assembler::createMarkerGraphSecondaryEdges,
             arg("secondaryEdgeMaxSkip"),
             arg("threadCount") = 0)
+        .def("clusterMarkerGraphEdgeOrientedReads",
+            &Assembler::clusterMarkerGraphEdgeOrientedReads,
+            arg("edgeId"),
+            arg("errorRateThreshold"),
+            arg("debug") = false)
+        .def("splitMarkerGraphSecondaryEdges",
+            &Assembler::splitMarkerGraphSecondaryEdges,
+            arg("errorRateThreshold"),
+            arg("minCoverage"),
+            arg("threadCount") = 0)
         .def("accessMarkerGraphEdges",
             &Assembler::accessMarkerGraphEdges,
             arg("accessEdgesReadWrite") = false,
             arg("accessConnectivityReadWrite") = false)
-            .def("transitiveReduction",
+        .def("transitiveReduction",
             &Assembler::transitiveReduction,
             arg("lowCoverageThreshold"),
             arg("highCoverageThreshold"),
@@ -615,9 +638,6 @@ PYBIND11_MODULE(shasta, module)
             arg("offDiagonalRatio"))
         .def("alignPseudoPaths",
             &Assembler::alignPseudoPaths)
-        .def("analyzeAssemblyGraphBubbles",
-            &Assembler::analyzeAssemblyGraphBubbles,
-            arg("debug") = true)
         .def("removeAssemblyGraph",
             &Assembler::removeAssemblyGraph)
 
@@ -626,21 +646,23 @@ PYBIND11_MODULE(shasta, module)
         // Assembly mode 2.
         .def("createAssemblyGraph2",
             &Assembler::createAssemblyGraph2,
-            arg("bubbleRemovalDiscordantRatioThreshold"),
-            arg("bubbleRemovalAmbiguityThreshold"),
-            arg("bubbleRemovalMaxPeriod"),
-            arg("superbubbleRemovalEdgeLengthThreshold"),
-            arg("phasingMinReadCount"),
-            arg("suppressGfaOutput"),
-            arg("suppressFastaOutput"),
-            arg("suppressDetailedOutput"),
-            arg("suppressPhasedOutput"),
-            arg("suppressHaploidOutput"),
+            arg("pruneLength"),
+            arg("mode2Options"),
+            arg("threadCount") = 0,
+            arg("debug") = false)
+
+        // Assembly mode 3.
+        .def("mode3Assembly",
+            &Assembler::mode3Assembly,
             arg("threadCount") = 0)
+        .def("accessMode3AssemblyGraph",
+            &Assembler::accessMode3AssemblyGraph)
+        .def("analyzeMode3Subgraph",
+            &Assembler::analyzeMode3Subgraph)
 
 
 
-            // Consensus caller.
+        // Consensus caller.
         .def("setupConsensusCaller",
             &Assembler::setupConsensusCaller)
 
@@ -661,7 +683,7 @@ PYBIND11_MODULE(shasta, module)
 
 
     // Expose class AssembledSegment to Python.
-    class_<AssembledSegment>(module, "AssembledSegment")
+    class_<AssembledSegment>(shastaModule, "AssembledSegment")
         .def("size", &AssembledSegment::size)
         .def("getBase", &AssembledSegment::getBase)
         .def("getRepeatCount", &AssembledSegment::getRepeatCount)
@@ -671,7 +693,7 @@ PYBIND11_MODULE(shasta, module)
 
 
     // Expose class CompressedCoverageData to Python.
-    class_<CompressedCoverageData>(module, "CompressedCoverageData")
+    class_<CompressedCoverageData>(shastaModule, "CompressedCoverageData")
         .def("getBase", &CompressedCoverageData::getBase)
         .def("getStrand", &CompressedCoverageData::getStrand)
         .def("getRepeatCount", &CompressedCoverageData::getRepeatCount)
@@ -681,50 +703,53 @@ PYBIND11_MODULE(shasta, module)
 
 
     // Constants.
-    module.attr("invalidGlobalMarkerGraphVertexId") = MarkerGraph::invalidVertexId;
-    module.attr("invalidCompressedGlobalMarkerGraphVertexId") =
+    shastaModule.attr("invalidGlobalMarkerGraphVertexId") = MarkerGraph::invalidVertexId;
+    shastaModule.attr("invalidCompressedGlobalMarkerGraphVertexId") =
         uint64_t(MarkerGraph::invalidCompressedVertexId);
 
 
 
     // Non-member functions exposed to Python.
-    module.def("testMultithreadedObject",
+    shastaModule.def("openPerformanceLog",
+        openPerformanceLog
+        );
+    shastaModule.def("testMultithreadedObject",
         testMultithreadedObject
         );
-    module.def("testMemoryMappedVector",
+    shastaModule.def("testMemoryMappedVector",
         testMemoryMappedVector
         );
-    module.def("testBase",
+    shastaModule.def("testBase",
         testBase
         );
-    module.def("testShortBaseSequence",
+    shastaModule.def("testShortBaseSequence",
         testShortBaseSequence
         );
-    module.def("testLongBaseSequence",
+    shastaModule.def("testLongBaseSequence",
         testLongBaseSequence
         );
-    module.def("testSplitRange",
+    shastaModule.def("testSplitRange",
         testSplitRange
         );
-    module.def("testCompactUndirectedGraph1",
+    shastaModule.def("testCompactUndirectedGraph1",
         testCompactUndirectedGraph1
         );
-    module.def("testCompactUndirectedGraph2",
+    shastaModule.def("testCompactUndirectedGraph2",
         testCompactUndirectedGraph1
         );
-    module.def("testSpoa",
+    shastaModule.def("testSpoa",
         testSpoa
         );
-    module.def("testSimpleBayesianConsensusCaller",
+    shastaModule.def("testSimpleBayesianConsensusCaller",
         testSimpleBayesianConsensusCaller
         );
-    module.def("testMedianConsensusCaller",
+    shastaModule.def("testMedianConsensusCaller",
         testMedianConsensusCaller
         );
-    module.def("testDeduplicateAndCount",
+    shastaModule.def("testDeduplicateAndCount",
         testDeduplicateAndCount
         );
-    module.def("dset64Test",
+    shastaModule.def("dset64Test",
         dset64Test,
         arg("n"),
         arg("m"),
@@ -732,23 +757,20 @@ PYBIND11_MODULE(shasta, module)
         arg("batchSize"),
         arg("seed")
         );
-    module.def("mappedCopy",
+    shastaModule.def("mappedCopy",
         mappedCopy
         );
-    module.def("testAlignmentCompression",
+    shastaModule.def("testAlignmentCompression",
         testAlignmentCompression
         );
-    module.def("testMap",
-        testMap
-        );
-    module.def("testMemoryMappedAllocator",
+    shastaModule.def("testMemoryMappedAllocator",
         MemoryMapped::testMemoryMappedAllocator
         );
-    module.def("testLapack",
+    shastaModule.def("testLapack",
         testLapack
         );
-    module.def("testShortestPathBoundedDistance",
-        testShortestPathBoundedDistance
+    shastaModule.def("testDiploidBayesianPhase",
+        testDiploidBayesianPhase
         );
 }
 
