@@ -1,6 +1,7 @@
 // Shasta.
 #include "ReadLoader.hpp"
 #include "computeRunLengthRepresentation.hpp"
+#include "performanceLog.hpp"
 #include "splitRange.hpp"
 using namespace shasta;
 
@@ -14,33 +15,25 @@ using namespace shasta;
 // Load reads from a fastq or fasta file.
 ReadLoader::ReadLoader(
     const string& fileName,
+    uint64_t representation, // 0 = raw sequence, 1 = RLE sequence
     uint64_t minReadLength,
     bool noCache,
     size_t threadCount,
     const string& dataNamePrefix,
     size_t pageSize,
-    bool detectPalindromesOnFastqLoad,
-    double qScoreRelativeMeanDifference,
-    double qScoreMinimumMean,
-    double qScoreMinimumVariance,
-    bool writePalindromesToCsv,
     Reads& reads):
     
     MultithreadedObject(*this),
     fileName(fileName),
+    representation(representation),
     minReadLength(minReadLength),
     noCache(noCache),
     threadCount(threadCount),
     dataNamePrefix(dataNamePrefix),
     pageSize(pageSize),
-    detectPalindromesOnFastqLoad(detectPalindromesOnFastqLoad),
-    qScoreRelativeMeanDifference(qScoreRelativeMeanDifference),
-    qScoreMinimumMean(qScoreMinimumMean),
-    qScoreMinimumVariance(qScoreMinimumVariance),
-    writePalindromesToCsv(writePalindromesToCsv),
     reads(reads)
 {
-    cout << timestamp << "Loading reads from " << fileName << endl;
+    performanceLog << timestamp << "Loading reads from " << fileName << endl;
 
     adjustThreadCount();
 
@@ -105,7 +98,7 @@ void ReadLoader::processFastaFile()
     const auto t3 = std::chrono::steady_clock::now();
 
 
-    cout << "Time to process this file:\n" <<
+    performanceLog << "Time to process this file:\n" <<
         "Allocate buffer + read: " << seconds(t1-t0) << " s.\n" <<
         "Parse: " << seconds(t2-t1) << " s.\n"
         "Store: " << seconds(t3-t2) << " s.\n"
@@ -254,14 +247,20 @@ void ReadLoader::processFastaFileThreadFunction(size_t threadId)
         }
 
         // Store the read bases.
-        if(computeRunLengthRepresentation(read, runLengthRead, readRepeatCount)) {
+        if(representation == 1) {
+            if(computeRunLengthRepresentation(read, runLengthRead, readRepeatCount)) {
+                thisThreadReadNames.appendVector(readName.begin(), readName.end());
+                thisThreadReadMetaData.appendVector(readMetaData.begin(), readMetaData.end());
+                thisThreadReads.append(runLengthRead);
+                thisThreadReadRepeatCounts.appendVector(readRepeatCount);
+            } else {
+                __sync_fetch_and_add(&discardedBadRepeatCountReadCount, 1);
+                __sync_fetch_and_add(&discardedBadRepeatCountBaseCount, read.size());
+            }
+        } else {
             thisThreadReadNames.appendVector(readName.begin(), readName.end());
             thisThreadReadMetaData.appendVector(readMetaData.begin(), readMetaData.end());
-            thisThreadReads.append(runLengthRead);
-            thisThreadReadRepeatCounts.appendVector(readRepeatCount);
-        } else {
-            __sync_fetch_and_add(&discardedBadRepeatCountReadCount, 1);
-            __sync_fetch_and_add(&discardedBadRepeatCountBaseCount, read.size());
+            thisThreadReads.append(read);
         }
     }
 
@@ -306,7 +305,7 @@ void ReadLoader::processFastqFile()
     // Find all line ends in the file.
     const auto t1 = std::chrono::steady_clock::now();
     findLineEnds();
-    cout << "Found " << lineEnds.size() << " lines in this file." << endl;
+    // cout << "Found " << lineEnds.size() << " lines in this file." << endl;
 
     // Check that the number of lines is a multiple of 4
     // (there must be exactly 4 mlines per read per the above assumptions).
@@ -330,7 +329,7 @@ void ReadLoader::processFastqFile()
     const auto t4 = std::chrono::steady_clock::now();
 
 
-    cout << "Time to process this file:\n" <<
+    performanceLog << "Time to process this file:\n" <<
         "Allocate buffer + read: " << seconds(t1-t0) << " s.\n" <<
         "Locate: " << seconds(t2-t1) << " s.\n"
         "Parse: " << seconds(t3-t2) << " s.\n"
@@ -338,18 +337,6 @@ void ReadLoader::processFastqFile()
         "Total: " << seconds(t4-t0) << " s." << endl;
 
 
-    // Write a csv file with the list of palindromic reads
-    // if shasta is running in "filterReads" mode.
-    // This should not too big as the typical rate of
-    // palindromic reads is around 1e-4.
-    if(writePalindromesToCsv) {
-        ofstream csvOut("PalindromicReads.csv");
-        for (const auto& item: threadPalindromicReadNames) {
-            for (const auto& name: item) {
-                csvOut << name << "\n";
-            }
-        }
-    }
 }
 
 
@@ -365,7 +352,6 @@ void ReadLoader::processFastqFileThreadFunction(size_t threadId)
     LongBaseSequences& thisThreadReads = *threadReads[threadId];
     MemoryMapped::VectorOfVectors<uint8_t, uint64_t>& thisThreadReadRepeatCounts =
         *threadReadRepeatCounts[threadId];
-    vector<string>& thisThreadPalindromicReadNames = threadPalindromicReadNames[threadId];
 
     // Find the total number of reads in the file.
     SHASTA_ASSERT((lineEnds.size() % 4) == 0); // We already checked for that.
@@ -492,35 +478,21 @@ void ReadLoader::processFastqFileThreadFunction(size_t threadId)
             continue;
         }
 
-        // Skip if the q scores have an obvious palindromic characteristic
-        if (detectPalindromesOnFastqLoad){
-            span<char> scores(scoresBegin, scoresEnd);
-            bool isPalindrome = isPalindromic(
-                    scores,
-                    qScoreRelativeMeanDifference,
-                    qScoreMinimumMean,
-                    qScoreMinimumVariance);
-
-            if (isPalindrome) {
-                __sync_fetch_and_add(&discardedPalindromicReadCount, 1);
-                __sync_fetch_and_add(&discardedPalindromicBaseCount, read.size());
-
-                // Update the running list of palindromic read names for this thread
-                thisThreadPalindromicReadNames.push_back(readName);
-
-                continue;
-            }
-        }
-
         // Store the read.
-        if(computeRunLengthRepresentation(read, runLengthRead, readRepeatCount)) {
+        if(representation == 1) {
+            if(computeRunLengthRepresentation(read, runLengthRead, readRepeatCount)) {
+                thisThreadReadNames.appendVector(readName.begin(), readName.end());
+                thisThreadReadMetaData.appendVector(readMetaData.begin(), readMetaData.end());
+                thisThreadReads.append(runLengthRead);
+                thisThreadReadRepeatCounts.appendVector(readRepeatCount);
+            } else {
+                __sync_fetch_and_add(&discardedBadRepeatCountReadCount, 1);
+                __sync_fetch_and_add(&discardedBadRepeatCountBaseCount, read.size());
+            }
+        } else {
             thisThreadReadNames.appendVector(readName.begin(), readName.end());
             thisThreadReadMetaData.appendVector(readMetaData.begin(), readMetaData.end());
-            thisThreadReads.append(runLengthRead);
-            thisThreadReadRepeatCounts.appendVector(readRepeatCount);
-        } else {
-            __sync_fetch_and_add(&discardedBadRepeatCountReadCount, 1);
-            __sync_fetch_and_add(&discardedBadRepeatCountBaseCount, read.size());
+            thisThreadReads.append(read);
         }
     }
 }
@@ -569,8 +541,8 @@ void ReadLoader::allocateBuffer()
 
     const auto t1 = std::chrono::steady_clock::now();
 
-    cout <<  "File size: " << fileSize << " bytes." << endl;
-    cout << "Buffer allocate time: " << seconds(t1 - t0) << " s." << endl;
+    performanceLog <<  "File size: " << fileSize << " bytes." << endl;
+    performanceLog << "Buffer allocate time: " << seconds(t1 - t0) << " s." << endl;
 }
 
 
@@ -612,8 +584,8 @@ bool ReadLoader::readFile(bool useODirect)
     const auto t1 = std::chrono::steady_clock::now();
     const double t01 = seconds(t1 - t0);
 
-    cout << "Read time: " << t01 << " s." << endl;
-    cout << "Read rate: " << double(fileSize) / t01 << " bytes/s." << endl;
+    performanceLog << "Read time: " << t01 << " s." << endl;
+    performanceLog << "Read rate: " << double(fileSize) / t01 << " bytes/s." << endl;
     return true;
 }
 
@@ -689,8 +661,7 @@ void ReadLoader::allocatePerThreadDataStructures()
     threadReadNames.resize(threadCount);
     threadReadMetaData.resize(threadCount);
     threadReads.resize(threadCount);
-    threadReadRepeatCounts.resize(threadCount);
-    threadPalindromicReadNames.resize(threadCount);
+    threadReadRepeatCounts.resize(threadCount); // Not actually used is representation==1
 }
 
 
@@ -706,8 +677,10 @@ void ReadLoader::allocatePerThreadDataStructures(size_t threadId)
     threadReads[threadId]->createNew(
         threadDataName(threadId, "Reads"), pageSize);
     threadReadRepeatCounts[threadId] = make_unique< MemoryMapped::VectorOfVectors<uint8_t, uint64_t> >();
-    threadReadRepeatCounts[threadId]->createNew(
-        threadDataName(threadId, "ReadRepeatCounts"), pageSize);
+    if(representation == 1) {
+        threadReadRepeatCounts[threadId]->createNew(
+            threadDataName(threadId, "ReadRepeatCounts"), pageSize);
+    }
 }
 
 
@@ -736,26 +709,34 @@ void ReadLoader::storeReads()
         // Access the repeat counts.
         MemoryMapped::VectorOfVectors<uint8_t, uint64_t>& thisThreadReadRepeatCounts =
             *threadReadRepeatCounts[threadId];
-        SHASTA_ASSERT(thisThreadReadRepeatCounts.size() == n);
+        if(representation == 1) {
+            SHASTA_ASSERT(thisThreadReadRepeatCounts.size() == n);
+        } else {
+            SHASTA_ASSERT(not thisThreadReadRepeatCounts.isOpen());
+        }
 
         // Store the reads.
         for(size_t i=0; i<n; i++) {
             reads.readNames.appendVector(thisThreadReadNames.begin(i), thisThreadReadNames.end(i));
             reads.readMetaData.appendVector(thisThreadReadMetaData.begin(i), thisThreadReadMetaData.end(i));
             reads.reads.append(thisThreadReads[i]);
-            const size_t j = reads.readRepeatCounts.size();
-            reads.readRepeatCounts.appendVector(thisThreadReadRepeatCounts.size(i));
-            copy(
-                thisThreadReadRepeatCounts.begin(i),
-                thisThreadReadRepeatCounts.end(i),
-                reads.readRepeatCounts.begin(j));
+            if(representation == 1) {
+                const size_t j = reads.readRepeatCounts.size();
+                reads.readRepeatCounts.appendVector(thisThreadReadRepeatCounts.size(i));
+                copy(
+                    thisThreadReadRepeatCounts.begin(i),
+                    thisThreadReadRepeatCounts.end(i),
+                    reads.readRepeatCounts.begin(j));
+            }
         }
 
         // Remove the data structures used by this thread.
         thisThreadReadNames.remove();
         thisThreadReadMetaData.remove();
         thisThreadReads.remove();
-        thisThreadReadRepeatCounts.remove();
+        if(representation == 1) {
+            thisThreadReadRepeatCounts.remove();
+        }
     }
 
     // Clear the per-thread data structures.
@@ -767,7 +748,9 @@ void ReadLoader::storeReads()
     // Free up unused allocated memory.
     reads.readNames.unreserve();
     reads.readMetaData.unreserve();
-    reads.readRepeatCounts.unreserve();
+    if(representation == 1) {
+        reads.readRepeatCounts.unreserve();
+    }
     reads.reads.unreserve();
 
     // Allocate enough space for readFlags which are populated later.
