@@ -10,6 +10,7 @@ using namespace shasta;
 
 
 void Reads::createNew(
+    uint64_t representationArgument, // 0 = raw sequence, 1 = RLE sequence
     const string& readsDataName,
     const string& readNamesDataName,
     const string& readMetaDataDataName,
@@ -18,15 +19,19 @@ void Reads::createNew(
     const string& readIdsSortedByNameDataName,
     uint64_t largeDataPageSize)
 {
+    representation = representationArgument;
     reads.createNew(readsDataName, largeDataPageSize);
     readNames.createNew(readNamesDataName, largeDataPageSize);
     readMetaData.createNew(readMetaDataDataName, largeDataPageSize);
-    readRepeatCounts.createNew(readRepeatCountsDataName, largeDataPageSize);
+    if(representation == 1) {
+        readRepeatCounts.createNew(readRepeatCountsDataName, largeDataPageSize);
+    }
     readFlags.createNew(readFlagsDataName, largeDataPageSize);
     readIdsSortedByName.createNew(readIdsSortedByNameDataName, largeDataPageSize);
 }
 
 void Reads::access(
+    uint64_t representationArgument, // 0 = raw sequence, 1 = RLE sequence
     const string& readsDataName,
     const string& readNamesDataName,
     const string& readMetaDataDataName,
@@ -34,10 +39,13 @@ void Reads::access(
     const string& readFlagsDataName,
     const string& readIdsSortedByNameDataName)
 {
+    representation = representationArgument;
     reads.accessExistingReadWrite(readsDataName);
     readNames.accessExistingReadWrite(readNamesDataName);
     readMetaData.accessExistingReadWrite(readMetaDataDataName);
-    readRepeatCounts.accessExistingReadWrite(readRepeatCountsDataName);
+    if(representation == 1) {
+        readRepeatCounts.accessExistingReadWrite(readRepeatCountsDataName);
+    }
     readFlags.accessExistingReadWrite(readFlagsDataName);
     readIdsSortedByName.accessExistingReadWrite(readIdsSortedByNameDataName);
 }
@@ -132,6 +140,7 @@ pair<Base, uint8_t> Reads::getOrientedReadBaseAndRepeatCount(
     OrientedReadId orientedReadId,
     uint32_t position) const
 {
+    SHASTA_ASSERT(representation == 1);
 
     // Extract the read id and strand.
     const ReadId readId = orientedReadId.getReadId();
@@ -164,21 +173,35 @@ vector<Base> Reads::getOrientedReadRawSequence(OrientedReadId orientedReadId) co
     // The sequence we will return;
     vector<Base> sequence;
 
-    // The number of bases stored, in run-length representation.
+    // The number of bases stored.
     const uint32_t storedBaseCount = uint32_t(reads[orientedReadId.getReadId()].baseCount);
 
+    if(representation == 1) {
 
-    // We are storing a run-length representation of the read.
-    // Expand it base by base to create the raw representation.
-    for(uint32_t position=0; position<storedBaseCount; position++) {
-        Base base;
-        uint8_t count;
-        tie(base, count) = getOrientedReadBaseAndRepeatCount(orientedReadId, position);
-        for(uint32_t i=0; i<uint32_t(count); i++) {
+        // We are storing a run-length representation of the read.
+        // Expand it base by base to create the raw representation.
+        for(uint32_t position=0; position<storedBaseCount; position++) {
+            Base base;
+            uint8_t count;
+            tie(base, count) = getOrientedReadBaseAndRepeatCount(orientedReadId, position);
+            for(uint32_t i=0; i<uint32_t(count); i++) {
+                sequence.push_back(base);
+            }
+        }
+
+    } else if(representation == 0) {
+
+        // We are storing the raw sequence of the read.
+        for(uint32_t position=0; position<storedBaseCount; position++) {
+            const Base base  = getOrientedReadBase(orientedReadId, position);
             sequence.push_back(base);
         }
+
+    } else {
+        SHASTA_ASSERT(0);
     }
 
+
     return sequence;
 }
 
@@ -187,17 +210,21 @@ vector<Base> Reads::getOrientedReadRawSequence(OrientedReadId orientedReadId) co
 // base a number of times equal to its repeat count.
 uint64_t Reads::getReadRawSequenceLength(ReadId readId) const
 {
-    // We are using the run-length representation.
-    // The number of raw bases equals the sum of all
-    // the repeat counts.
-    // Don't use std::accumulate to compute the sum,
-    // otherwise the sum is computed using uint8_t!
-    const auto& counts = readRepeatCounts[readId];
-    uint64_t sum = 0;;
-    for(uint8_t count: counts) {
-        sum += count;
+    if(representation == 1) {
+        // We are using the run-length representation.
+        // The number of raw bases equals the sum of all
+        // the repeat counts.
+        // Don't use std::accumulate to compute the sum,
+        // otherwise the sum is computed using uint8_t!
+        const auto& counts = readRepeatCounts[readId];
+        uint64_t sum = 0;;
+        for(uint8_t count: counts) {
+            sum += count;
+        }
+        return sum;
+    } else {
+        return reads[readId].baseCount;
     }
-    return sum;
 }
 
 
@@ -462,8 +489,6 @@ void Reads::writeReadLengthHistogram(const string& fileName) {
         SHASTA_ASSERT(cumulativeBaseCount == 0);
     }
 
-    cout << "See " << fileName << " and Binned-" << fileName <<
-        " for details of the read length distribution." << endl;
 }
 
 
