@@ -12,7 +12,9 @@
 #include "ConfigurationTable.hpp"
 #include "Coverage.hpp"
 #include "filesystem.hpp"
+#include "performanceLog.hpp"
 #include "Reads.hpp"
+#include "Tee.hpp"
 #include "timestamp.hpp"
 #include "platformDependent.hpp"
 #include "SimpleBayesianConsensusCaller.hpp"
@@ -34,11 +36,11 @@ namespace shasta {
             Assembler&,
             const AssemblerOptions&,
             uint32_t threadCount);
-        void mode1Assembly(
+        void mode2Assembly(
             Assembler&,
             const AssemblerOptions&,
             uint32_t threadCount);
-        void mode2Assembly(
+        void mode3Assembly(
             Assembler&,
             const AssemblerOptions&,
             uint32_t threadCount);
@@ -54,7 +56,7 @@ namespace shasta {
         void segmentFaultHandler(int);
 
         // Functions that implement --command keywords
-        void assemble(const AssemblerOptions&);
+        void assemble(const AssemblerOptions&, int argumentCount, const char** arguments);
         void saveBinaryData(const AssemblerOptions&);
         void cleanupBinaryData(const AssemblerOptions&);
         void createBashCompletionScript(const AssemblerOptions&);
@@ -71,13 +73,15 @@ namespace shasta {
             "cleanupBinaryData",
             "createBashCompletionScript",
             "explore",
-            "filterReads",
             "listCommands",
             "listConfiguration",
             "listConfigurations",
             "saveBinaryData"};
 
     }
+
+    Tee tee;
+    ofstream shastaLog;
 }
 using namespace shasta;
 
@@ -161,19 +165,9 @@ void shasta::main::main(int argumentCount, const char** arguments)
 
 
     // Execute the requested command.
-    if(assemblerOptions.commandLineOnlyOptions.command == "assemble" or
-        assemblerOptions.commandLineOnlyOptions.command == "filterReads") {
-
-        // For assemblies we also echo out the command line options.
-        cout << timestamp << "Assembly begins with the following command line:" << endl;
-        for(int i=0; i<argumentCount; i++) {
-            cout << arguments[i] << " ";
-        }
-        cout << endl;
-
-        assemble(assemblerOptions);
+    if(assemblerOptions.commandLineOnlyOptions.command == "assemble") {
+        assemble(assemblerOptions, argumentCount, arguments);
         return;
-
     } else if(assemblerOptions.commandLineOnlyOptions.command == "cleanupBinaryData") {
         cleanupBinaryData(assemblerOptions);
         return;
@@ -184,7 +178,7 @@ void shasta::main::main(int argumentCount, const char** arguments)
 #ifdef SHASTA_HTTP_SERVER
         explore(assemblerOptions);
 #else
-        throw runtime_error("Shasta was not built with HTTP server support.");
+        throw runtime_error("This Shasta build does not include HTTP server support.");
 #endif
         return;
     } else if(assemblerOptions.commandLineOnlyOptions.command == "createBashCompletionScript") {
@@ -211,10 +205,10 @@ void shasta::main::main(int argumentCount, const char** arguments)
 
 // Implementation of --command assemble.
 void shasta::main::assemble(
-    const AssemblerOptions& assemblerOptions)
+    const AssemblerOptions& assemblerOptions,
+    int argumentCount, const char** arguments)
 {
-    SHASTA_ASSERT(assemblerOptions.commandLineOnlyOptions.command == "assemble" or
-        assemblerOptions.commandLineOnlyOptions.command == "filterReads");
+    SHASTA_ASSERT(assemblerOptions.commandLineOnlyOptions.command == "assemble");
 
 
     // Various checks for option validity.
@@ -301,7 +295,7 @@ void shasta::main::assemble(
 
 
 
-    // Create the run output directory. If it exists and is not empty then stop.
+    // Create the assembly directory. If it exists and is not empty then stop.
     bool exists = std::filesystem::exists(assemblerOptions.commandLineOnlyOptions.assemblyDirectory);
     bool isDir = std::filesystem::is_directory(assemblerOptions.commandLineOnlyOptions.assemblyDirectory);
     if (exists) {
@@ -323,9 +317,27 @@ void shasta::main::assemble(
     } else {
         filesystem::createDirectory(assemblerOptions.commandLineOnlyOptions.assemblyDirectory);
     }
-    // Make the output directory current.
+
+    // Make the assembly directory current.
     filesystem::changeDirectory(assemblerOptions.commandLineOnlyOptions.assemblyDirectory);
 
+    // Open the performance log.
+    openPerformanceLog("performance.log");
+    performanceLog << timestamp << "Assembly begins." << endl;
+
+    // Open stdout.log and "tee" (duplicate) stdout to it.
+    if(not assemblerOptions.commandLineOnlyOptions.suppressStdoutLog) {
+        shastaLog.open("stdout.log");
+        tee.duplicate(cout, shastaLog);
+    }
+
+    // Echo out the command line options.
+    cout << timestamp << "Assembly begins.\nCommand line:" << endl;
+    for(int i=0; i<argumentCount; i++) {
+        cout << arguments[i] << " ";
+    }
+    cout << endl;
+
 
 
     // Set up the run directory as required by the memoryMode and memoryBacking options.
@@ -368,7 +380,10 @@ void shasta::main::assemble(
 #endif
 
     // Create the Assembler.
-    Assembler assembler(dataDirectory, true, pageSize);
+    Assembler assembler(dataDirectory, true, assemblerOptions.readsOptions.representation, pageSize);
+    assembler.assemblerInfo->readGraphCreationMethod = assemblerOptions.readGraphOptions.creationMethod;
+    assembler.assemblerInfo->assemblyMode = assemblerOptions.assemblyOptions.mode;
+
 
     // Run the assembly.
     assemble(assembler, assemblerOptions, inputFileAbsolutePaths);
@@ -395,6 +410,8 @@ void shasta::main::assemble(
     // Write out the build id again.
     cout << buildId() << endl;
 
+    performanceLog << timestamp << "Assembly ends." << endl;
+    cout << timestamp << "Assembly ends." << endl;
 }
 
 
@@ -529,40 +546,19 @@ void shasta::main::assemble(
 
 
     // Add reads from the specified input files.
-    cout << timestamp << "Begin loading reads from " << inputFileNames.size() << " files." << endl;
+    performanceLog << timestamp << "Begin loading reads from " << inputFileNames.size() << " files." << endl;
     const auto t0 = steady_clock::now();
     for(const string& inputFileName: inputFileNames) {
-        // Only dump the palindromes to a separate csv if running in filter mode.
-        // Otherwise ReadSummary.csv already contains the palindromic reads that were identified
-        // by self alignment (but not the palindromes identified by Q score)
-        bool writeToCsv = (assemblerOptions.commandLineOnlyOptions.command == "filterReads");
 
         assembler.addReads(
             inputFileName,
             assemblerOptions.readsOptions.minReadLength,
             assemblerOptions.readsOptions.noCache,
-            assemblerOptions.readsOptions.palindromicReads.detectOnFastqLoad,
-            assemblerOptions.readsOptions.palindromicReads.qScoreRelativeMeanDifference,
-            assemblerOptions.readsOptions.palindromicReads.qScoreMinimumMean,
-            assemblerOptions.readsOptions.palindromicReads.qScoreMinimumVariance,
-            writeToCsv,
             threadCount);
     }
 
     if(assembler.getReads().readCount() == 0) {
-        // There could be no reads remaining at this point, but it does not constitute an error
-        // if running in filter mode
-        if(assemblerOptions.commandLineOnlyOptions.command == "filterReads") {
-            // Write the assembly summary.
-            ofstream html("AssemblySummary.html");
-            assembler.writeAssemblySummary(html, true);
-            ofstream json("AssemblySummary.json");
-            assembler.writeAssemblySummaryJson(json, true);
-
-            return;
-        } else{
-            throw runtime_error("There are no input reads.");
-        }
+        throw runtime_error("There are no input reads.");
     }
     
 
@@ -598,8 +594,8 @@ void shasta::main::assemble(
     assembler.histogramReadLength("ReadLengthHistogram.csv");
 
     const auto t1 = steady_clock::now();
-    cout << timestamp << "Done loading reads from " << inputFileNames.size() << " files." << endl;
-    cout << "Read loading took " << seconds(t1-t0) << "s." << endl;
+    performanceLog << timestamp << "Done loading reads from " << inputFileNames.size() << " files." << endl;
+    performanceLog << "Read loading took " << seconds(t1-t0) << "s." << endl;
 
 
 
@@ -657,50 +653,12 @@ void shasta::main::assemble(
             "Specify a value between 0 and 4, inclusive.");
     }
 
-#if 0
-    if(not assemblerOptions.kmersOptions.file.empty() or
-        assemblerOptions.kmersOptions.file[0] != '/') {
-
-        // A file name was specified. Read the k-mers to be used as markers from there.
-
-        // This must be an absolute path.
-        if(assemblerOptions.kmersOptions.file[0] != '/') {
-            throw runtime_error("Option --Kmers.file must specify an absolute path. "
-                "A relative path is not accepted.");
-        }
-
-        // Read the k-mers.
-        assembler.readKmersFromFile(
-            assemblerOptions.kmersOptions.k,
-            assemblerOptions.kmersOptions.file);
-
-
-    } else if(assemblerOptions.kmersOptions.suppressHighFrequencyMarkers) {
-
-        // Randomly select the k-mers to be used as markers, but
-        // excluding those that are highly frequent in the input reads.
-        assembler.selectKmersBasedOnFrequency(
-            assemblerOptions.kmersOptions.k,
-            assemblerOptions.kmersOptions.probability, 231,
-            assemblerOptions.kmersOptions.enrichmentThreshold, threadCount);
-    } else {
-
-        // Randomly select the k-mers to be used as markers.
-        assembler.randomlySelectKmers(
-            assemblerOptions.kmersOptions.k,
-            assemblerOptions.kmersOptions.probability, 231);
-    }
-#endif
 
 
     // Find the markers in the reads.
     assembler.findMarkers(0);
 
     if(!assemblerOptions.readsOptions.palindromicReads.skipFlagging) {
-        // Only dump the palindromes to a separate csv if running in filter mode.
-        // Otherwise ReadSummary.csv already contains the palindromic reads that were identified
-        // by self alignment (but not the palindromes identified by Q score)
-        bool writeToCsv = (assemblerOptions.commandLineOnlyOptions.command == "filterReads");
 
         // Flag palindromic reads.
         // These will be excluded from further processing.
@@ -711,29 +669,10 @@ void shasta::main::assemble(
             assemblerOptions.readsOptions.palindromicReads.alignedFractionThreshold,
             assemblerOptions.readsOptions.palindromicReads.nearDiagonalFractionThreshold,
             assemblerOptions.readsOptions.palindromicReads.deltaThreshold,
-            writeToCsv,
             threadCount);
     }
 
 
-    if(assemblerOptions.commandLineOnlyOptions.command == "filterReads") {
-        ofstream csv("PassingReads.csv");
-        csv << "ReadId,ReadName\n";
-
-        const auto& reads = assembler.getReads();
-
-        for (ReadId readId=0; readId<reads.readCount(); readId++) {
-            csv << readId << ',' << reads.getReadName(readId) << '\n';
-        }
-
-        // Write the assembly summary.
-        ofstream html("AssemblySummary.html");
-        assembler.writeAssemblySummary(html, true);
-        ofstream json("AssemblySummary.json");
-        assembler.writeAssemblySummaryJson(json, true);
-
-        return;
-    }
 
     // Find alignment candidates.
     if(assemblerOptions.minHashOptions.allPairs) {
@@ -789,6 +728,15 @@ void shasta::main::assemble(
             assemblerOptions.readGraphOptions.maxAlignmentCount,
             assemblerOptions.alignOptions.maxTrim);
 
+        // Actual alignment criteria are as specified in the command line options
+        // and/or configuration.
+        assembler.assemblerInfo->actualMinAlignedFraction = assemblerOptions.alignOptions.minAlignedFraction;
+        assembler.assemblerInfo->actualMinAlignedMarkerCount = assemblerOptions.alignOptions.minAlignedMarkerCount;
+        assembler.assemblerInfo->actualMaxDrift = assemblerOptions.alignOptions.maxDrift;
+        assembler.assemblerInfo->actualMaxSkip = assemblerOptions.alignOptions.maxSkip;
+        assembler.assemblerInfo->actualMaxTrim = assemblerOptions.alignOptions.maxTrim;
+
+
     } else if(assemblerOptions.readGraphOptions.creationMethod == 2) {
         assembler.createReadGraph2(
             assemblerOptions.readGraphOptions.maxAlignmentCount,
@@ -842,15 +790,15 @@ void shasta::main::assemble(
     case 0:
         mode0Assembly(assembler, assemblerOptions, threadCount);
         break;
-    case 1:
-        mode1Assembly(assembler, assemblerOptions, threadCount);
-        break;
     case 2:
         mode2Assembly(assembler, assemblerOptions, threadCount);
         break;
+    case 3:
+        mode3Assembly(assembler, assemblerOptions, threadCount);
+        break;
     default:
         throw runtime_error("Invalid value specified for --Assembly.mode. "
-            "Valid values are 0 (default) and 1 or 2 (experimental), but " +
+            "Valid values are 0 (haploid assembly) and 2 (phased diploid assembly), but " +
             to_string(assemblerOptions.assemblyOptions.mode) +
             " was specified.");
     }
@@ -885,13 +833,15 @@ void shasta::main::assemble(
     ofstream htmlIndex("index.html");
     assembler.writeAssemblyIndex(htmlIndex);
 
-    cout << timestamp << endl;
-    cout << "Assembly time statistics:\n"
+    performanceLog << timestamp << endl;
+    performanceLog << "Assembly time statistics:\n"
         "    Elapsed seconds: " << elapsedTime << "\n"
         "    Elapsed minutes: " << elapsedTime/60. << "\n"
         "    Elapsed hours:   " << elapsedTime/3600. << "\n";
-    cout << "Average CPU utilization: " << averageCpuUtilization << endl;
-    cout << "Peak Memory usage (bytes): " << peakMemoryUsage << endl;
+    performanceLog << "Average CPU utilization: " << averageCpuUtilization << endl;
+    performanceLog << "Peak Memory usage: " << peakMemoryUsage << " bytes = " <<
+        int(std::round(double(peakMemoryUsage) / (1024. * 1024. * 1024.)) ) << " GiB" << endl;
+
 }
 
 
@@ -1006,39 +956,6 @@ void shasta::main::mode0Assembly(
 
 
 
-    // If marker graph refinement was requested, do it now, then regenerate
-    // marker graph edges.
-    if(assemblerOptions.markerGraphOptions.refineThreshold > 0) {
-        assembler.refineMarkerGraph(
-            assemblerOptions.markerGraphOptions.refineThreshold,
-            threadCount);
-
-        // This destroyed everything except the vertices.
-        // Recreate the edges and redo all of the above steps.
-
-        // Find the reverse complement of each marker graph vertex.
-        assembler.findMarkerGraphReverseComplementVertices(threadCount);
-
-        // Create edges of the marker graph.
-        assembler.createMarkerGraphEdges(threadCount);
-        assembler.findMarkerGraphReverseComplementEdges(threadCount);
-
-        // Approximate transitive reduction.
-        assembler.transitiveReduction(
-            assemblerOptions.markerGraphOptions.lowCoverageThreshold,
-            assemblerOptions.markerGraphOptions.highCoverageThreshold,
-            assemblerOptions.markerGraphOptions.maxDistance,
-            assemblerOptions.markerGraphOptions.edgeMarkerSkipThreshold);
-        if(assemblerOptions.markerGraphOptions.reverseTransitiveReduction) {
-            assembler.reverseTransitiveReduction(
-                assemblerOptions.markerGraphOptions.lowCoverageThreshold,
-                assemblerOptions.markerGraphOptions.highCoverageThreshold,
-                assemblerOptions.markerGraphOptions.maxDistance);
-        }
-    }
-
-
-
     // Prune the marker graph.
     assembler.pruneMarkerGraphStrongSubgraph(
         assemblerOptions.markerGraphOptions.pruneIterationCount);
@@ -1092,7 +1009,9 @@ void shasta::main::mode0Assembly(
     }
 
     // Compute optimal repeat counts for each vertex of the marker graph.
-    assembler.assembleMarkerGraphVertices(threadCount);
+    if(assemblerOptions.readsOptions.representation == 1) {
+        assembler.assembleMarkerGraphVertices(threadCount);
+    }
 
     // If coverage data was requested, compute and store coverage data for the vertices.
     if(assemblerOptions.assemblyOptions.storeCoverageData or
@@ -1131,127 +1050,6 @@ void shasta::main::mode0Assembly(
 
 
 
-void shasta::main::mode1Assembly(
-    Assembler& assembler,
-    const AssemblerOptions& assemblerOptions,
-    uint32_t threadCount)
-{
-    // Create marker graph vertices.
-    assembler.createMarkerGraphVertices(
-        assemblerOptions.markerGraphOptions.minCoverage,
-        assemblerOptions.markerGraphOptions.maxCoverage,
-        assemblerOptions.markerGraphOptions.minCoveragePerStrand,
-        assemblerOptions.markerGraphOptions.allowDuplicateMarkers,
-        assemblerOptions.markerGraphOptions.peakFinderMinAreaFraction,
-        assemblerOptions.markerGraphOptions.peakFinderAreaStartIndex,
-        threadCount);
-    assembler.findMarkerGraphReverseComplementVertices(threadCount);
-
-    // Create marker graph edges.
-    // For assembly mode 1 we use createMarkerGraphEdgesStrict
-    // with minimum edge coverage (total and per strand).
-    assembler.createMarkerGraphEdgesStrict(
-        assemblerOptions.markerGraphOptions.minEdgeCoverage,
-        assemblerOptions.markerGraphOptions.minEdgeCoveragePerStrand, threadCount);
-    assembler.findMarkerGraphReverseComplementEdges(threadCount);
-
-    // Coverage histograms for vertices and edges of the marker graph.
-    assembler.computeMarkerGraphCoverageHistogram();
-
-    // Create the assembly graph.
-    assembler.createAssemblyGraphEdges();
-    assembler.createAssemblyGraphVertices();
-
-    // Analyze bubbles in the assembly graph.
-    assembler.analyzeAssemblyGraphBubbles(true);
-
-    // Create a new read graph, using the bubble analysis
-    // to exclude alignments.
-    assembler.createReadGraphMode1(
-        assemblerOptions.readGraphOptions.maxAlignmentCount);
-
-    // Compute connected components of the read graph.
-    // These are currently not used.
-    assembler.computeReadGraphConnectedComponents();
-
-
-
-    // Create a new marker graph and assembly graph using this new read graph.
-
-    // Remove the old marker graph and assembly graph.
-    assembler.removeMarkerGraph();
-    assembler.removeAssemblyGraph();
-
-    // For the second step we use lower coverage thresholds
-    // for marker graph vertices and edges.
-    // ******* EXPOSE THESE WHEN CODE STABILIZES (also in Mode1Assembly.py)
-    const uint64_t minVertexCoverageFinal = 3;
-    const uint64_t minVertexCoveragePerStrandFinal = 0;
-    const uint64_t minEdgeCoverageFinal = 2;
-    const uint64_t minEdgeCoveragePerStrandFinal = 0;
-
-
-    // Create marker graph vertices.
-    assembler.createMarkerGraphVertices(
-        minVertexCoverageFinal,
-        assemblerOptions.markerGraphOptions.maxCoverage,
-        minVertexCoveragePerStrandFinal,
-        assemblerOptions.markerGraphOptions.allowDuplicateMarkers,
-        assemblerOptions.markerGraphOptions.peakFinderMinAreaFraction,
-        assemblerOptions.markerGraphOptions.peakFinderAreaStartIndex,
-        threadCount);
-    assembler.findMarkerGraphReverseComplementVertices(threadCount);
-
-    // Create marker graph edges.
-    // For assembly mode 1 we use createMarkerGraphEdgesStrict
-    // with minimum edge coverage (total and per strand).
-    assembler.createMarkerGraphEdgesStrict(
-        minEdgeCoverageFinal,
-        minEdgeCoveragePerStrandFinal, threadCount);
-    assembler.findMarkerGraphReverseComplementEdges(threadCount);
-
-    // To recovery contiguity, add secondary edges.
-    const uint32_t secondaryEdgeMaxSkip = 1000000;    // ********* EXPOSE WHEN CODE STABILIZES (also in Mode1Assembly.py)
-    assembler.createMarkerGraphSecondaryEdges(secondaryEdgeMaxSkip, threadCount);
-
-    // Coverage histograms for vertices and edges of the marker graph.
-    assembler.computeMarkerGraphCoverageHistogram();
-
-    // Simplify the marker graph to remove bubbles and superbubbles.
-    // The maxLength parameter controls the maximum number of markers
-    // for a branch to be collapsed during each iteration.
-    assembler.simplifyMarkerGraph(assemblerOptions.markerGraphOptions.simplifyMaxLengthVector, false);
-
-    // Create the assembly graph.
-    assembler.createAssemblyGraphEdges();
-    assembler.createAssemblyGraphVertices();
-    assembler.writeGfa1BothStrandsNoSequence("Assembly-BothStrands-NoSequence.gfa");
-
-    // Compute optimal repeat counts for each vertex of the marker graph.
-    assembler.assembleMarkerGraphVertices(threadCount);
-
-    // Compute consensus sequence for marker graph edges to be used for assembly.
-    assembler.assembleMarkerGraphEdges(
-        threadCount,
-        assemblerOptions.assemblyOptions.markerGraphEdgeLengthThresholdForConsensus,
-        assemblerOptions.assemblyOptions.storeCoverageData or
-        assemblerOptions.assemblyOptions.storeCoverageDataCsvLengthThreshold>0,
-        false
-        );
-
-    // Use the assembly graph for sequence assembly.
-    assembler.assemble(
-        threadCount,
-        assemblerOptions.assemblyOptions.storeCoverageDataCsvLengthThreshold);
-    assembler.computeAssemblyStatistics();
-    assembler.writeGfa1("Assembly.gfa");
-    assembler.writeGfa1BothStrands("Assembly-BothStrands.gfa");
-    assembler.writeFasta("Assembly.fasta");
-
-}
-
-
-
 void shasta::main::mode2Assembly(
     Assembler& assembler,
     const AssemblerOptions& assemblerOptions,
@@ -1280,14 +1078,21 @@ void shasta::main::mode2Assembly(
     assembler.computeMarkerGraphCoverageHistogram();
 
     // To recover contiguity, add secondary edges.
-    const uint32_t secondaryEdgeMaxSkip = 1000000;    // ********* EXPOSE WHEN CODE STABILIZES (also in Mode1Assembly.py)
-    assembler.createMarkerGraphSecondaryEdges(secondaryEdgeMaxSkip, threadCount);
+    assembler.createMarkerGraphSecondaryEdges(
+        uint32_t(assemblerOptions.markerGraphOptions.secondaryEdgesMaxSkip),
+        threadCount);
+    assembler.splitMarkerGraphSecondaryEdges(
+        assemblerOptions.markerGraphOptions.secondaryEdgesSplitErrorRateThreshold,
+        assemblerOptions.markerGraphOptions.secondaryEdgesSplitMinCoverage,
+        threadCount);
 
     // Coverage histograms for vertices and edges of the marker graph.
     assembler.computeMarkerGraphCoverageHistogram();
 
     // Compute optimal repeat counts for each vertex of the marker graph.
-    assembler.assembleMarkerGraphVertices(threadCount);
+    if(assemblerOptions.readsOptions.representation == 1) {
+        assembler.assembleMarkerGraphVertices(threadCount);
+    }
 
     // Compute consensus sequence for all marker graph edges.
     assembler.assembleMarkerGraphEdges(
@@ -1300,16 +1105,66 @@ void shasta::main::mode2Assembly(
 
     // Create the mode 2 assembly graph.
     assembler.createAssemblyGraph2(
-        assemblerOptions.assemblyOptions.bubbleRemovalDiscordantRatioThreshold,
-        assemblerOptions.assemblyOptions.bubbleRemovalAmbiguityThreshold,
-        assemblerOptions.assemblyOptions.bubbleRemovalMaxPeriod,
-        assemblerOptions.assemblyOptions.superbubbleRemovalEdgeLengthThreshold,
-        assemblerOptions.assemblyOptions.phasingMinReadCount,
-        assemblerOptions.assemblyOptions.suppressGfaOutput,
-        assemblerOptions.assemblyOptions.suppressFastaOutput,
-        assemblerOptions.assemblyOptions.suppressDetailedOutput,
-        assemblerOptions.assemblyOptions.suppressPhasedOutput,
-        assemblerOptions.assemblyOptions.suppressHaploidOutput,
+        assemblerOptions.assemblyOptions.pruneLength,
+        assemblerOptions.assemblyOptions.mode2Options,
+        threadCount, false);
+
+
+}
+
+
+
+
+
+
+void shasta::main::mode3Assembly(
+    Assembler& assembler,
+    const AssemblerOptions& assemblerOptions,
+    uint32_t threadCount)
+{
+    // Create marker graph vertices.
+    assembler.createMarkerGraphVertices(
+        assemblerOptions.markerGraphOptions.minCoverage,
+        assemblerOptions.markerGraphOptions.maxCoverage,
+        assemblerOptions.markerGraphOptions.minCoveragePerStrand,
+        assemblerOptions.markerGraphOptions.allowDuplicateMarkers,
+        assemblerOptions.markerGraphOptions.peakFinderMinAreaFraction,
+        assemblerOptions.markerGraphOptions.peakFinderAreaStartIndex,
+        threadCount);
+    assembler.findMarkerGraphReverseComplementVertices(threadCount);
+
+    // Create marker graph edges.
+    // For assembly mode 1 we use createMarkerGraphEdgesStrict
+    // with minimum edge coverage (total and per strand).
+    assembler.createMarkerGraphEdgesStrict(
+        assemblerOptions.markerGraphOptions.minEdgeCoverage,
+        assemblerOptions.markerGraphOptions.minEdgeCoveragePerStrand, threadCount);
+    assembler.findMarkerGraphReverseComplementEdges(threadCount);
+
+    // Coverage histograms for vertices and edges of the marker graph.
+    assembler.computeMarkerGraphCoverageHistogram();
+
+    // In mode 3 assembly, we don't add secondary edges.
+
+    // Coverage histograms for vertices and edges of the marker graph.
+    assembler.computeMarkerGraphCoverageHistogram();
+
+    // Compute optimal repeat counts for each vertex of the marker graph.
+    if(assemblerOptions.readsOptions.representation == 1) {
+        assembler.assembleMarkerGraphVertices(threadCount);
+    }
+
+    // Compute consensus sequence for all marker graph edges.
+    assembler.assembleMarkerGraphEdges(
+        threadCount,
+        assemblerOptions.assemblyOptions.markerGraphEdgeLengthThresholdForConsensus,
+        assemblerOptions.assemblyOptions.storeCoverageData or
+        assemblerOptions.assemblyOptions.storeCoverageDataCsvLengthThreshold>0,
+        true
+        );
+
+    // Run mode 3 assembly.
+    assembler.mode3Assembly(
         threadCount);
 
 
@@ -1468,7 +1323,7 @@ void shasta::main::explore(
     }
     
     // Create the Assembler.
-    Assembler assembler("Data/", false, 0);
+    Assembler assembler("Data/", false, 1, 0);
     
     // Access all available binary data.
     assembler.accessAllSoft();
@@ -1588,7 +1443,7 @@ void shasta::main::createBashCompletionScript(const AssemblerOptions& assemblerO
 void shasta::main::listCommands()
 {
     cout << "Valid commands are:" << endl;
-    for(const string command: commands) {
+    for(const string& command: commands) {
         cout << command << endl;
     }
 }
@@ -1597,12 +1452,15 @@ void shasta::main::listCommands()
 
 void shasta::main::listConfigurations()
 {
-    cout << "Valid Shasta built-in configurations are:" << endl;
+    cout << "Valid Shasta built-in configurations, in chronological order, are:\n" << endl;
     for(const auto& p: configurationTable) {
         cout << p.first << endl;
     }
-    cout << "In the command line, after --config, "
-        "you can specify any of the above configuration names, "
+    cout <<
+        "\nUse \"shasta --command listConfiguration --config configurationName\" "
+        "to list the details of one of the above configurations.\n\n"
+        "When running an assembly, you can use option \"--config\" "
+        "to specify any of the above configuration names, "
         "or the name of a configuration file. "
         "See shasta/conf for examples of configuration files. "
         "Each of the above configurations has a corresponding "
@@ -1619,16 +1477,15 @@ void shasta::main::listConfiguration(const AssemblerOptions& options)
         throw runtime_error("Specify --config with a valid configuration name.");
     }
 
-    auto it = configurationTable.find(configName);
-    if(it == configurationTable.end()) {
+    const string* configuration = getConfiguration(configName);
+    if(configuration == 0) {
         const string message = configName + " is not a valid configuration name.";
         cout << message << endl;
         listConfigurations();
         throw runtime_error(configName);
     }
 
-    const string& configuration = it->second;
-    cout << configuration << flush;
+    cout << *configuration << flush;
 }
 
 
